""" Test cases for autofs responder

:requirement: Ldap Provider - automount
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""

from __future__ import print_function
import re
import subprocess
import time
import pytest
from sssd.testlib.common.utils import sssdTools, LdapOperations

JOURNALCTL_CMD = "journalctl -x -n 50 --no-pager"


def restart_autofs(multihost):
    for _ in range(1, 20):
        cmd = multihost.client[0].run_command("systemctl restart autofs", raiseonerr=False).returncode
        if cmd == 0:
            break
        time.sleep(10)
    else:
        raise Exception("autofs restart failed too many times")


@pytest.mark.usefixtures("setup_sssd", "create_posix_usersgroups",
                         "enable_autofs_schema", "enable_autofs_service")
@pytest.mark.automount
class Testautofsresponder(object):
    """ Autofs responder test cases

        :setup:
          1. Configure nfs server exporting /export
          2. Load autofs maps to Directory Server under
             ou=automout,dc=example,dc=test
          3. Add an nisMapEntry in automount specifying nfs server,
             directory and file system(nfs)
          4. Join RHEL7 client to Windows AD using realm
    """
    @pytest.mark.parametrize('add_nisobject', ['/export'], indirect=True)
    @pytest.mark.tier1
    def test_001_searchbasedn(self, multihost, add_nisobject):
        """
        :title: IDM-SSSD-TC: LDAP-Provider: Automount: Verify automount rules
         are searched from basedn
        :id: f0b8962d-446b-412e-959a-41182d906dbf
        :steps:
          1. Edit sssd.conf and specify autofs_provider = ldap and restart
             autofs
          2. Access /export share
        :expectedresults:
          1. Should succeed
          2. /export share should be mounted successfully
        """
        # pylint: disable=unused-argument
        multihost.master[0].run_command(['touch', '/export/nfs-test'])
        for service in ['sssd', 'autofs']:
            srv = 'systemctl restart %s' % service
            try:
                multihost.client[0].run_command(srv)
            except subprocess.CalledProcessError:
                pytest.fail("Unable to start %s service" % service)
            time.sleep(5)
        try:
            multihost.client[0].run_command(['automount', '-m'])
        except subprocess.CalledProcessError:
            pytest.fail("automount -m command failed")
        nfs_test = 'ls -l /export/nfs-test'
        cmd = multihost.client[0].run_command(nfs_test, raiseonerr=False)
        assert cmd.returncode == 0

    @pytest.mark.parametrize('add_nisobject', ['/export'], indirect=True)
    @pytest.mark.tier1
    def test_002_offline(self, multihost, add_nisobject):
        """
        :title: IDM-SSSD-TC: LDAP-Provider Automount: Verify automount
         maps are retrieved from cache when sssd is offline
        :id: 5e85710a-bd86-416b-82aa-10f254421381
        :steps:
          1. Edit sssd.conf and specify autofs_provider = ldap
          2. Restart Autofs
          3. pkill -USR1 sssd
          4. access /export/nfs-test
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. sssd should be offline
          4. /export/nfs-test share should be accessible from client
        """
        multihost.master[0].run_command(['touch', '/export/nfs-test'])
        for service in ['sssd', 'autofs']:
            srv = 'systemctl restart %s' % service
            try:
                multihost.client[0].run_command(srv)
            except subprocess.CalledProcessError:
                pytest.fail("Unable to start %s service" % service)
        time.sleep(30)
        automount = 'automount -m'
        try:
            multihost.client[0].run_command(automount)
        except subprocess.CalledProcessError:
            pytest.xfail("automount -m command failed")
        pkill_cmd = 'pkill -USR1 sssd'
        cmd = multihost.client[0].run_command(pkill_cmd, raiseonerr=False)
        assert cmd.returncode == 0
        try:
            multihost.client[0].run_command(automount)
        except subprocess.CalledProcessError:
            pytest.fail("automount -m command failed")
        nfstest = 'ls -l /export/nfs-test'
        cmd = multihost.client[0].run_command(nfstest, raiseonerr=False)
        assert cmd.returncode == 0

    @pytest.mark.parametrize('add_nisobject', ['/export'], indirect=True)
    @pytest.mark.tier1
    def test_003_setbasedn(self, multihost, set_autofs_search_base,
                           add_nisobject):
        """
        :title: IDM-SSSD-TC: LDAP-Provider Automount: Verify automount
         rules are searched when ldap_autofs_search_base is set
        :id: 76e7449e-226a-47c5-bc7f-0bb6b9e749e7
        :steps:
          1. Edit sssd.conf and specify below parameters:
             autofs_provider = ldap
             ldap_autofs_search_base = ou=automount,dc=example,dc=test
          2. Restart sssd
          3. Execute automount -m
          4. Access /export shared
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. automount -m should succeed and show maps from AD
          4. client is able mount nfs share /export
        """
        # pylint: disable=unused-argument
        multihost.master[0].run_command(['touch', '/export/nfs-test'])
        for service in ['sssd', 'autofs']:
            srv = 'systemctl restart %s' % service
            try:
                multihost.client[0].run_command(srv)
            except subprocess.CalledProcessError:
                multihost.client[0].run_command(JOURNALCTL_CMD)
                pytest.fail("Unable to start %s service" % service)
        time.sleep(30)
        try:
            multihost.client[0].run_command(['automount', '-m'])
        except subprocess.CalledProcessError:
            pytest.fail("automount -m command failed")
        nfs_test = 'ls -l /export/nfs-test'
        cmd = multihost.client[0].run_command(nfs_test, raiseonerr=False)
        assert cmd.returncode == 0

    @pytest.mark.parametrize('add_nisobject', ['/export'], indirect=True)
    @pytest.mark.tier1
    def test_004_autofsnone(self, multihost, set_autofs_search_base,
                            add_nisobject):
        """
        :title: IDM-SSSD-TC: LDAP-Provider Automount: Verify maps are
         loaded from cache and maps are accessible when autofs_provider is None
        :id: 67daac8b-c7c2-4dca-8331-5e95fd21b483
        :steps:
          1. Set autofs_provider = None
          2. Restart sssd service
          3. Execute automount -m
          4. Run ls -l /export/nfs-share
        :expectedresults:
          1. Should succeed
          2. sssd service should be started successfully
          3. automount -m should execute successfully
          4. /export/nfs-share should be accessible
        """
        multihost.master[0].run_command(['touch', '/export/nfs-test'])
        for service in ['sssd', 'autofs']:
            srv = 'systemctl restart %s' % service
            try:
                multihost.client[0].run_command(srv)
            except subprocess.CalledProcessError:
                pytest.fail("Unable to start %s service" % service)
        time.sleep(30)
        try:
            multihost.client[0].run_command(['automount', '-m'])
        except subprocess.CalledProcessError:
            pytest.fail("automount -m command failed")
        nfs_test = 'ls -l /export/nfs-test'
        cmd = multihost.client[0].run_command(nfs_test, raiseonerr=False)
        assert cmd.returncode == 0
        client = sssdTools(multihost.client[0])
        domain_name = client.get_domain_section_name()
        section = 'domain/{}'.format(domain_name)
        sssd_params = {'autofs_provider': 'none'}
        client.sssd_conf(section, sssd_params)
        srv = 'systemctl restart sssd'
        try:
            multihost.client[0].run_command(srv)
        except subprocess.CalledProcessError:
            pytest.fail("Unable to start sssd")
        time.sleep(10)
        try:
            multihost.client[0].run_command(['automount', '-m'])
        except subprocess.CalledProcessError:
            pytest.fail("automount -m command failed")
        cmd = multihost.client[0].run_command(nfs_test, raiseonerr=False)
        assert cmd.returncode == 0

    @pytest.mark.tier1_2
    def test_two_automount_maps(self, multihost,
                                backupsssdconf):
        """
        :title: Automount sssd issue when 2 maps have same key in
         different case
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1873715
        :id: d28e6eec-ac9f-11eb-b0f5-002b677efe14
        :customerscenario: true
        :steps:
            1. Configure SSSD with autofs, automountMap,
               automount, automountInformation
            2. Add 2 automount entries in LDAP with
               same key ( cn: MIT and cn: mit)
            3. We should have the 2 automounts working
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
        """
        tools = sssdTools(multihost.client[0])
        domain_name = tools.get_domain_section_name()
        client = sssdTools(multihost.client[0])
        domain_params = {'services': 'nss, pam, autofs'}
        client.sssd_conf('sssd', domain_params)
        domain_params = {
            'ldap_autofs_map_object_class': 'automountMap',
            'ldap_autofs_map_name': 'ou',
            'ldap_autofs_entry_object_class': 'automount',
            'ldap_autofs_entry_key': 'cn',
            'ldap_autofs_entry_value': 'automountInformation'}
        client.sssd_conf(f'domain/{domain_name}', domain_params)
        multihost.client[0].service_sssd('restart')
        share_list = ['/export', '/export1', '/export2']
        nfs_server_ip = multihost.master[0].ip
        client_ip = multihost.client[0].ip
        server = sssdTools(multihost.master[0])
        bkup = 'cp -af /etc/exports /etc/exports.backup'
        multihost.master[0].run_command(bkup)
        server.export_nfs_fs(share_list, client_ip)
        search = multihost.master[0].run_command("grep 'fsid=0' "
                                                 "/etc/exports")
        if search.returncode == 0:
            multihost.master[0].run_command("sed -i 's/,fsid=0//g' "
                                            "/etc/exports")
        start_nfs = 'systemctl start nfs-server'
        multihost.master[0].run_command(start_nfs)
        ldap_uri = 'ldap://%s' % (multihost.master[0].sys_hostname)
        ds_rootdn = 'cn=Directory Manager'
        ds_rootpw = 'Secret123'
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        for ou_ou in ['auto.master', 'auto.direct', 'auto.home']:
            user_info = {'ou': f'{ou_ou}'.encode('utf-8'),
                         'objectClass': [b'top', b'automountMap']}
            user_dn = f'ou={ou_ou},dc=example,dc=test'
            (_, _) = ldap_inst.add_entry(user_info, user_dn)
        user_info = {'cn': '/-'.encode('utf-8'),
                     'objectClass': [b'top', b'automount'],
                     'automountInformation': 'auto.direct'.encode('utf-8')}
        user_dn = 'cn=/-,ou=auto.master,dc=example,dc=test'
        (_, _) = ldap_inst.add_entry(user_info, user_dn)
        user_info = {'cn': '/home'.encode('utf-8'),
                     'objectClass': [b'top', b'automount'],
                     'automountInformation': 'auto.home'.encode('utf-8')}
        user_dn = 'cn=/home,ou=auto.master,dc=example,dc=test'
        (_, _) = ldap_inst.add_entry(user_info, user_dn)
        user_info = {'cn': 'MIT'.encode('utf-8'),
                     'objectClass': [b'top', b'automount']}
        user_dn = f'automountinformation={nfs_server_ip}:/export1,' \
                  f'ou=auto.home,dc=example,dc=test'
        (_, _) = ldap_inst.add_entry(user_info, user_dn)
        user_info = {'cn': 'mit'.encode('utf-8'),
                     'objectClass': [b'top', b'automount']}
        user_dn = f'automountinformation={nfs_server_ip}:/export2,' \
                  f'ou=auto.home,dc=example,dc=test'
        (_, _) = ldap_inst.add_entry(user_info, user_dn)
        multihost.client[0].run_command("systemctl stop sssd ; "
                                        "rm -rf /var/log/sssd/* ; "
                                        "rm -rf /var/lib/sss/db/* ; "
                                        "systemctl start sssd")
        time.sleep(10)
        restart_autofs(multihost)
        multihost.client[0].run_command("automount -m")
        multihost.master[0].run_command("touch /export1/export1")
        multihost.master[0].run_command("touch /export2/export2")
        time.sleep(2)
        MIT_export = multihost.client[0].run_command("ls /home/MIT")
        mit_export = multihost.client[0].run_command("ls /home/mit")
        restore = 'cp -af /etc/exports.backup /etc/exports'
        multihost.master[0].run_command(restore)
        multihost.client[0].run_command("systemctl stop autofs", raiseonerr=False)
        stop_nfs = 'systemctl stop nfs-server'
        multihost.master[0].run_command(stop_nfs)
        for dn_dn in [f'automountinformation={nfs_server_ip}:/export1,'
                      f'ou=auto.home,dc=example,dc=test',
                      f'automountinformation={nfs_server_ip}:/export2,'
                      f'ou=auto.home,dc=example,dc=test',
                      'cn=/-,ou=auto.master,dc=example,dc=test',
                      'cn=/home,ou=auto.master,dc=example,dc=test',
                      'ou=auto.master,dc=example,dc=test',
                      'ou=auto.direct,dc=example,dc=test',
                      'ou=auto.home,dc=example,dc=test']:
            multihost.master[0].run_command(f'ldapdelete -x -D '
                                            f'"cn=Directory Manager" '
                                            f'-w Secret123 -H ldap:// {dn_dn}')
        assert 'export1' in MIT_export.stdout_text
        assert 'export2' in mit_export.stdout_text

    @pytest.mark.parametrize('add_nisobject', ['/export'], indirect=True)
    @pytest.mark.tier2
    def test_005_autofsnotset(self, multihost, set_autofs_search_base,
                              add_nisobject):
        """
        :title: IDM-SSSD-TC: LDAP-Provider: Automount: Verify automount maps
         are loaded from Directory Server when autofs provider is not set
         in sssd.conf
        :id: 3d93a51c-332c-4a88-af68-71a92e8a1b8e
        :steps:
          1. Access nfs share /export/nfs-test with autofs provider not set
        :expectedresults:
          1. Verify automount maps are loaded from AD and client is able to
             mount nfs share
        """
        # pylint: disable=unused-argument
        client = sssdTools(multihost.client[0])
        domain_name = client.get_domain_section_name()
        section = 'domain/{}'.format(domain_name)
        sssd_params = {'autofs_provider': 'none'}
        client.sssd_conf(section, sssd_params, action='delete')
        multihost.master[0].run_command(['touch', '/export/nfs-test'])
        for service in ['sssd', 'autofs']:
            restart = 'systemctl restart %s' % service
            cmd = multihost.client[0].run_command(restart, raiseonerr=False)
            if cmd.returncode != 0:
                multihost.client[0].run_command(JOURNALCTL_CMD)
                pytest.fail("Failed to restart %s" % service)
        automount = 'automount -m'
        cmd = multihost.client[0].run_command(automount, raiseonerr=False)
        assert cmd.returncode == 0
        nfstest = 'ls -l /export/nfs-test'
        cmd = multihost.client[0].run_command(nfstest, raiseonerr=False)
        assert cmd.returncode == 0

    @pytest.mark.parametrize('add_nisobject', ['/project1'], indirect=True)
    @pytest.mark.tier2
    def test_006_updatedmaps(self, multihost, set_autofs_search_base,
                             add_nisobject):
        """
        :title: IDM-SSSD-TC: LDAP-Provider Automount: Verify sssd properly
         updates cache when automount maps are updated with more entries
         in Directory server
        :id: 0509a594-bd7a-45d6-b40b-4efcdc6b27f0
        :steps:
          1. Add /project1 entry to auto.direct map
          2. Restart autofs
          3. Access /project1 nfs share
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Verify /project1 can be mounted on nfs client
        """
        # pylint: disable=unused-argument
        nfstest = '/project1/project1-test'
        create_nfsfile = 'touch %s' % nfstest
        multihost.master[0].run_command(create_nfsfile)
        for service in ['sssd', 'autofs']:
            restart = 'systemctl restart %s' % service
            cmd = multihost.client[0].run_command(restart, raiseonerr=False)
            if cmd.returncode != 0:
                multihost.client[0].run_command(JOURNALCTL_CMD)
                pytest.fail("Failed to restart %s" % service)
        automount = 'automount -m'
        cmd = multihost.client[0].run_command(automount, raiseonerr=False)
        assert cmd.returncode == 0
        nfstest = 'ls -l %s' % nfstest
        cmd = multihost.client[0].run_command(nfstest, raiseonerr=False)
        assert cmd.returncode == 0

    @pytest.mark.tier2
    def test_007_1206221(self, multihost, set_dslimits, indirect_nismaps):
        """
        :title: automount: sssd should not always read
         entire autofs map from ldap
        :id: e79723ce-ad26-44ae-82ff-1afaae22f188
        :customerscenario: True
        :steps:
          1. Add Indirect map auto.idmtest which has mount point keys
             from foo1 to foo20 pointing to /projects/foo1 to /projects/foo20
          2. Set sizelimit to 10 on Directory server.
        :expectedresults:
          1. Should succeed
          2. Verify all the map keys are accessible
        """
        for service in ['sssd', 'autofs']:
            restart = 'systemctl restart %s' % service
            cmd = multihost.client[0].run_command(restart, raiseonerr=False)
            if cmd.returncode != 0:
                multihost.client[0].run_command(JOURNALCTL_CMD)
                pytest.fail("Failed to restart %s" % service)
        count = 0
        for i in range(1, 20):
            path = '/idmtest/foo%d' % (i)
            list_dir = 'ls -l %s' % (path)
            cmd = multihost.client[0].run_command(list_dir, raiseonerr=False)
            if cmd.returncode != 0:
                count += 1
        assert count == 0

    @pytest.mark.tier2
    def test_008_wildcardsearch(self, multihost, indirect_nismaps,
                                set_ldap_uri):
        """
        :title: automount: sssd should not use wildcard
         search to fetch map keys
        :id: 92640015-52b9-4e76-9e63-ea7357eec9cd
        :steps:
          1. Add Indirect map auto.idmtest which has mount point keys
             from foo1 to foo20 pointing to /projects/foo1 to /projects/foo20
        :expectedresults:
          1. Verify sssd doesn't use (cn=*)(objectclass=nisObject)
        """
        auto_pcapfile = '/tmp/automount.pcap'
        ldap_host = multihost.master[0].sys_hostname
        tcpdump_cmd = 'tcpdump -s0 host %s -w %s' % (ldap_host, auto_pcapfile)
        multihost.client[0].run_command(tcpdump_cmd, bg=True)
        # pid_cmd = 'pidof tcpdump'
        # pid = multihost.client[0].run_command(pid_cmd, raiseonerr=False)
        for service in ['sssd', 'autofs']:
            restart = 'systemctl restart %s' % service
            cmd = multihost.client[0].run_command(restart, raiseonerr=False)
            if cmd.returncode != 0:
                multihost.client[0].run_command(JOURNALCTL_CMD)
                pytest.fail("Failed to restart %s" % service)
        count = 0
        for i in range(1, 20):
            path = '/idmtest/foo%d' % (i)
            list_dir = 'ls -l %s' % (path)
            cmd = multihost.client[0].run_command(list_dir, raiseonerr=False)
            if cmd.returncode != 0:
                count += 1
        assert count == 0
        kill_cmd = 'pkill tcpdump'
        multihost.client[0].run_command(kill_cmd)
        # get the pcap file in text format
        conv_text = 'tshark -r %s -R ldap.filter'\
                    ' -V -2 > /tmp/automount.txt' % (auto_pcapfile)
        multihost.client[0].run_command(conv_text, raiseonerr=False)
        # get the file
        multihost.client[0].transport.get_file('/tmp/automount.txt',
                                               '/tmp/automount.txt')
        with open('/tmp/automount.txt', 'r') as outfile:
            tcpdump_ascii = outfile.read()
        for i in range(1, 10):
            key = 'foo%d' % i
            ldap_filter = r'\(\&\(cn=%s\)\(objectclass=nisObject\)\)' % key
            log_1 = re.compile(r'%s' % (ldap_filter))
            assert log_1.search(tcpdump_ascii)

        # delete the pcap file
        del_pcap = 'rm -f %s' % auto_pcapfile
        multihost.client[0].run_command(del_pcap)

    @pytest.mark.parametrize('add_nisobject', ['/export'], indirect=True)
    @pytest.mark.tier2
    def test_009_maps_after_coming_online(self, multihost, add_nisobject):
        """
        :title: fetch autofs map after coming online from offline
        :id: b9da6e0e-3d8b-4465-b435-338708d0d51e
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1113639
        :customerscenario: True
        :steps:
          1. edit sssd.conf and specify autofs_provider = ad
          2. restart autofs
          3. firewalld block 389 and 636
          4. stop sssd, autofs.
          5. remove sssd cache
          6. Start sssd
          7. remove firewall rule
          8. start autofs
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
          4. Should succeed
          5. Should succeed
          6. Should succeed
          7. Should succeed
          8. Should succeed
        """
        multihost.master[0].run_command(['touch', '/export/nfs-test'])
        client = sssdTools(multihost.client[0])
        domain_name = client.get_domain_section_name()
        for service in ['sssd', 'autofs']:
            client.service_ctrl("stop", service)
        client.service_ctrl("start", "firewalld")
        multihost.client[0].run_command("iptables -A "
                                        "OUTPUT -p tcp "
                                        "--dport 636 -j DROP")
        multihost.client[0].run_command("iptables -A "
                                        "OUTPUT -p tcp "
                                        "--dport 389 -j DROP")
        client.clear_sssd_cache()
        time.sleep(5)
        cmdy = 'id foo1@%s' % domain_name
        multihost.client[0].run_command(cmdy, raiseonerr=False)
        cmd = 'sssctl domain-status %s' % domain_name
        cmd1 = multihost.client[0].run_command(cmd, raiseonerr=False)
        find = re.compile(r'Online status: Offline')
        result = find.search(cmd1.stdout_text)
        assert result is not None
        client.firewall_port(636, 'OPEN')
        client.firewall_port(389, 'OPEN')
        client.firewall_port('ALL', 'delall')
        client.service_ctrl("stop", "firewalld")
        multihost.client[0].run_command("iptables -F")
        time.sleep(60)
        cmd2 = client.service_ctrl("start", "autofs")
        assert cmd2 == 0

    @pytest.mark.parametrize('add_nisobject', ['/export'], indirect=True)
    @pytest.mark.tier1
    def test_010_delay_in_unknown_mnt_pt_lookup_error(self, multihost,
                                                      add_nisobject):
        """
        :title: IDM-SSSD-TC: LDAP-Provider: Automount: autofs lookups for
         unknown mounts are delayed for 50s
        :id: 74cf1842-a8a0-4b6c-9670-82d73eb8ec6d
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=2013218
        :customerscenario: true
        :steps:
          1. Edit sssd.conf and specify autofs_provider = ldap and restart
             autofs
          2. Access /export share
          3. Access Non-existen mount point
        :expectedresults:
          1. Should succeed
          2. /export share should be mounted successfully
          3. Error for Non-existent share should appear immediately
        """
        multihost.master[0].run_command(['touch', '/export/nfs-test'])
        for service in ['sssd', 'autofs']:
            srv = 'systemctl restart %s' % service
            try:
                multihost.client[0].run_command(srv)
            except subprocess.CalledProcessError:
                pytest.fail("Unable to start %s service" % service)
            time.sleep(5)
        try:
            multihost.client[0].run_command(['automount', '-m'])
        except subprocess.CalledProcessError:
            pytest.fail("automount -m command failed")
        nfs_test = 'stat /export/nfs-test'
        cmd = multihost.client[0].run_command(nfs_test, raiseonerr=False)
        assert cmd.returncode == 0
        nfs_test = 'time stat /export/non_existing_nfs'
        tm1 = time.time()
        cmd = multihost.client[0].run_command(nfs_test, raiseonerr=False)
        tm2 = time.time()
        assert cmd.returncode != 0 and tm2 - tm1 <= 4



"""
IPA Trusts.

:requirement: IDM-SSSD-REQ: Testing SSSD in IPA Provider
"""

from __future__ import annotations

import pytest
from sssd_test_framework.roles.generic import GenericADProvider
from sssd_test_framework.roles.ipa import IPA
from sssd_test_framework.topology import KnownTopologyGroup


@pytest.mark.importance("low")
@pytest.mark.ticket(jira="RHEL-3925", gh=6942)
@pytest.mark.topology(KnownTopologyGroup.IPATrust)
def test_ipa_trusts__lookup_group_without_sid(ipa: IPA, trusted: GenericADProvider):
    """
    :title: Subdomain stays online if IPA group is missing SID
    :description: This test is to check a bug that made SSSD go offline when an expected attribute was missing.
        This happens during applying overrides on cached group during initgroups of trusted user. If the group
        does not have SID (it's GID is outside the sidgen range), SSSD goes offline.
    :setup:
        1. Create IPA external group "external-group" and add AD user "Administrator" as a member
        2. Create IPA posix group "posix-group" and add "external-group" as a member
        3. Clear SSSD cache and logs on IPA server
        4. Restart SSSD on IPA server
    :steps:
        1. Lookup AD administrator user
        2. Clear user cache
        3. Lookup AD administrator user
        4. Check logs using sssctl for domain status
    :expectedresults:
        1. User is found and is a member of 'posix-group'
        2. User cache expired
        3. User is found and is a member of 'posix-group'
        4. No messages indicating AD went offline
    :customerscenario: True
    """
    username = trusted.fqn("administrator")
    external = ipa.group("external-group").add(external=True).add_member(username)
    ipa.group("posix-group").add(gid=5001).add_member(external)

    ipa.sssd.clear(db=True, memcache=True, logs=True)
    ipa.sssd.restart()

    # Cache trusted user
    result = ipa.tools.id(username)
    assert result is not None, "User not found!"
    assert result.memberof("posix-group"), "User is not a member of 'posix-group'!"

    # Expire the user and resolve it again, this will trigger the affected code path
    ipa.sssctl.cache_expire(user=username)
    result = ipa.tools.id(username)
    assert result is not None, "User not found!"
    assert result.memberof("posix-group"), "User is not a member of 'posix-group'!"

    # Check that SSSD did not go offline
    status = ipa.sssctl.domain_status(trusted.domain, online=True)
    assert "online status: offline" not in status.stdout.lower(), "AD domain went offline!"
    assert "online status: online" in status.stdout.lower(), "AD domain was not online!"



"""Constants related to file location and log location"""

SSSD_DEFAULT_CONF = '/etc/sssd/sssd.conf'
KRB_DEFAULT_CONF = '/etc/krb5.conf'
RESOLVE_DEFAULT_CONF = '/etc/resolv.conf'
SMB_DEFAULT_CONF = '/etc/samba/smb.conf'
JOURNALD_DEFAULT_CONF = '/etc/systemd/journald.conf'
NSSWITCH_DEFAULT_CONF = '/etc/nsswitch.conf'
SYSCONFIG_SSSD = '/etc/sysconfig/sssd'
NSLCD_DEFAULT_CONF = '/etc/nslcd.conf'
PAM_LDAP_CONF = '/etc/pam_ldap.conf'
SYSCONFIG_NETWORK = '/etc/sysconfig/network'
NFS_EXPORT_FILE = '/etc/exports'
KRB_KDC_DATA_DIR = '/var/kerberos/krb5kdc'
KRB_DEFAULT_KEYTAB = '/etc/krb5.keytab'
KRB_ADMIN_KEYTAB = '%s/kadm5.keytab' % (KRB_KDC_DATA_DIR)
KRB_ACL_FILE = '%s/kadm5.acl' % (KRB_KDC_DATA_DIR)
KRB_KDC_CONF = '%s/kdc.conf' % (KRB_KDC_DATA_DIR)
KRB_LOG_FILE = '/var/log/krb5kdc.log'
KRB_ADMIN_LOG = '/var/log/kadmind.log'
SSSD_DB_CACHE_DIR = '/var/lib/sss/db'
SSSD_MC_CACHE_DIR = '/var/lib/sss/mc'
SSSD_CONFIG_LDB = '%s/config.ldb' % (SSSD_DB_CACHE_DIR)
SSSD_LDB = '%s/sssd.ldb' % (SSSD_DB_CACHE_DIR)
SSSD_PROXY_LDAP = '/etc/pam.d/sssdproxyldap'
LDAP_CONF = '/etc/openldap/ldap.conf'
ETC_HOSTS = '/etc/hosts'
SSSD_LOG_DIR = '/var/log/sssd'
SECURE_LOG_DIR = '/var/log/secure'
SMB_LOG_DIR = '/var/log/samba'



""" Automation of nsaccount lock

:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
:requirement: ns_account_lock
:casecomponent: sssd
"""

from __future__ import print_function
import time
import ldap
import pytest
from sssd.testlib.common.utils import LdapOperations
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.ssh2_python import check_login_client


def execute_cmd(multihost, command):
    """ Execute command on client """
    cmd = multihost.client[0].run_command(command)
    return cmd


def lock_check(multihost, user):
    """Check if user is locked"""
    out_put = execute_cmd(multihost,
                          'grep "Performing RHDS access '
                          'check for user" /var/log/sssd/'
                          'sssd_example1.log').stdout_text
    assert f"Performing RHDS access check " \
           f"for user [{user}@example1]" in out_put
    assert "pam_sss(sshd:account): system info:" \
           " [The user account is locked on the server]" \
           in execute_cmd(multihost,
                          'grep "pam_sss(sshd:account): '
                          'system info" /var/log/secure').stdout_text
    assert f"pam_sss(sshd:account):" \
           f" Access denied for user" \
           f" {user}@example1: 6 (Permission denied)"\
           in execute_cmd(multihost, 'grep '
                                     '"Access denied '
                                     'for user" '
                                     '/var/log/secure').stdout_text


def unlock_check(multihost, user):
    """Check if user is unlocked"""
    assert f"Performing RHDS access " \
           f"check for user [{user}@example1]" \
           in execute_cmd(multihost,
                          'grep "Performing RHDS '
                          'access check for user"'
                          ' /var/log/sssd/sssd_example1.log').stdout_text
    assert f"{user}@example1" in \
           execute_cmd(multihost,
                       'grep "is not locked." '
                       '/var/log/sssd/sssd_example1.log').stdout_text


def manage_user_roles(multihost, user, lock, type1):
    """Manage users and roles"""
    master_e = multihost.master[0].ip
    assert f"Entry {user},ou=people," \
           f"dc=example,dc=test is {lock}" in \
           execute_cmd(multihost, f"dsidm "
                                  f"ldap://{master_e}:389 -D"
                                  f" 'cn=Directory Manager'"
                                  f" -w Secret123 -b dc=example,"
                                  f"dc=test {type1}"
                                  f" {lock} {user},ou=people,"
                                  f"dc=example,dc=test").stdout_text


def clean_sys(multihost):
    """Clean logs and restart"""
    tools = sssdTools(multihost.client[0])
    execute_cmd(multihost, "rm -vf /var/log/sssd/*")
    execute_cmd(multihost, "> /var/log/secure")
    tools.clear_sssd_cache()


@pytest.mark.tier1_2
@pytest.mark.usefixtures('setup_sssd_krb',
                         'create_posix_usersgroups',
                         'ns_account_lock')
@pytest.mark.nsaccountlock
class TestNsAccountLock(object):
    """
    This is for ns_account automation
    """
    @staticmethod
    def test_user_inactivated_locked(multihost):
        """
        :title: User is inactivated or locked
        :id: 5787bb3e-3045-11ec-8da7-845cf3eff344
        :steps:
            1. Check user is present
            2. Lock user, check user is really locked.
            3. Unlock user, check user is really unlocked.
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
        """
        clean_sys(multihost)
        assert "foo1@example1" in \
               execute_cmd(multihost, "getent -s sss passwd"
                                      " foo1@example1").stdout_text
        assert "ldapusers@example1" in \
               execute_cmd(multihost, "getent -s sss group "
                                      "ldapusers@example1").stdout_text

        manage_user_roles(multihost, "uid=foo1", "lock", "account")
        with pytest.raises(Exception):
            check_login_client(multihost, "foo1@example1", 'Secret123')
        time.sleep(3)
        lock_check(multihost, "foo1")
        # User is activated or unlocked
        clean_sys(multihost)
        manage_user_roles(multihost, "uid=foo1", "unlock", "account")
        check_login_client(multihost, "foo1@example1", 'Secret123')
        time.sleep(3)
        unlock_check(multihost, "foo1")

    @staticmethod
    def test_inactive_managed_roles(multihost):
        """
        title: Inactive managed roles
        :id: 4f685ee0-3045-11ec-b3f8-845cf3eff344
        :steps:
            1. Make managed role it inactive
            2. User added to the above inactive managed role
            3. User removed from the above inactive managed role
            4. Activate managed role
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
            4. Should succeed
        """
        clean_sys(multihost)
        client = sssdTools(multihost.client[0])
        ldap_uri = f'ldap://{multihost.master[0].ip}'
        ds_rootdn = 'cn=Directory Manager'
        ds_rootpw = 'Secret123'
        manage_user_roles(multihost, "cn=managed", "lock", "role")
        with pytest.raises(Exception):
            check_login_client(multihost, "foo1@example1", 'Secret123')
        time.sleep(3)
        lock_check(multihost, "foo1")
        # User added to the above inactive managed role
        clean_sys(multihost)
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        user_dn = 'uid=foo2,ou=People,dc=example,dc=test'
        role_dn = "cn=managed,ou=people,dc=example,dc=test"
        add_member = [(ldap.MOD_ADD, 'nsRoleDN', role_dn.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(user_dn, add_member)
        assert ret == 'Success'
        with pytest.raises(Exception):
            check_login_client(multihost, "foo2@example1", 'Secret123')
        time.sleep(3)
        lock_check(multihost, "foo2")
        # User removed from the above inactive managed role
        clean_sys(multihost)
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        user_dn = 'uid=foo2,ou=People,dc=example,dc=test'
        role_dn = "cn=managed,ou=people,dc=example,dc=test"
        add_member = [(ldap.MOD_DELETE, 'nsRoleDN', role_dn.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(user_dn, add_member)
        assert ret == 'Success'
        check_login_client(multihost, "foo2@example1", 'Secret123')
        time.sleep(3)
        unlock_check(multihost, "foo2")
        # Activate managed role
        clean_sys(multihost)
        manage_user_roles(multihost, "cn=managed", "unlock", "role")
        check_login_client(multihost, "foo1@example1", 'Secret123')
        time.sleep(3)
        unlock_check(multihost, "foo1")

    @staticmethod
    def test_inactivated_filtered_roles(multihost):
        """
        title: Inactivated filtered roles
        :id: 4286dac6-3045-11ec-8fd0-845cf3eff344
        :steps:
            1. Make filter role inactive
            2. User added to the above inactive filtered role
            3. User removed from the above inactive filtered role
            4. Activate filtered role
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
            4. Should succeed
        """
        clean_sys(multihost)
        client = sssdTools(multihost.client[0])
        ldap_uri = f'ldap://{multihost.master[0].ip}'
        ds_rootdn = 'cn=Directory Manager'
        ds_rootpw = 'Secret123'
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        user_dn = 'uid=foo3,ou=People,dc=example,dc=test'
        role_dn = "filtered"
        add_member = [(ldap.MOD_ADD, 'o', role_dn.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(user_dn, add_member)
        assert ret == 'Success'
        manage_user_roles(multihost, "cn=filtered", "lock", "role")
        with pytest.raises(Exception):
            check_login_client(multihost, "foo3@example1", 'Secret123')

        time.sleep(3)
        lock_check(multihost, "foo3")
        # User added to the above inactive filtered role
        clean_sys(multihost)
        with pytest.raises(Exception):
            check_login_client(multihost, "foo4@example1", 'Secret123')

        time.sleep(3)
        lock_check(multihost, "foo4")
        # User removed from the above inactive filtered role
        clean_sys(multihost)
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        user_dn = 'uid=foo3,ou=People,dc=example,dc=test'
        role_dn = "filtered"
        add_member = [(ldap.MOD_DELETE, 'o', role_dn.encode('utf-8'))]
        (ret, _) = ldap_inst.modify_ldap(user_dn, add_member)
        assert ret == 'Success'
        check_login_client(multihost, "foo3@example1", 'Secret123')
        time.sleep(3)
        unlock_check(multihost, "foo3")
        # Activate filtered role
        clean_sys(multihost)
        manage_user_roles(multihost, "cn=filtered", "unlock", "role")
        check_login_client(multihost, "foo4@example1", 'Secret123')
        time.sleep(3)
        unlock_check(multihost, "foo4")

    @staticmethod
    def test_nested_role_inactivated(multihost):
        """
        title: Nested role has both the above roles and inactivated
        :id: 312e42c8-3045-11ec-88d4-845cf3eff344
        :steps:
            1. Add nasted role and make it inactive
            2. Nested role has the managed role
            3. Nested role has the filtered role
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
        """
        clean_sys(multihost)
        client = sssdTools(multihost.client[0])
        master_e = multihost.master[0].ip
        ldap_uri = f'ldap://{master_e}'
        ds_rootdn = 'cn=Directory Manager'
        ds_rootpw = 'Secret123'
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        user_info = {
            'cn': 'nested'.encode('utf-8'),
            'objectClass': [b'top',
                            b'LdapSubEntry',
                            b'nsRoleDefinition',
                            b'nsComplexRoleDefinition',
                            b'nsNestedRoleDefinition'],
            'nsRoleDN': [b'cn=filtered,ou=people,dc=example,dc=test',
                         b'cn=managed,ou=people,dc=example,dc=test']
        }
        user_dn = 'cn=nested,ou=People,dc=example,dc=test'
        (_, _) = ldap_inst.add_entry(user_info, user_dn)
        manage_user_roles(multihost, "cn=nested", "lock", "role")
        with pytest.raises(Exception):
            check_login_client(multihost, "foo1@example1", 'Secret123')

        time.sleep(3)
        lock_check(multihost, "foo1")
        with pytest.raises(Exception):
            check_login_client(multihost, "foo4@example1", 'Secret123')

        time.sleep(3)
        lock_check(multihost, "foo4")
        # Nested role has both the above roles and activated
        clean_sys(multihost)
        ldap_inst = LdapOperations(ldap_uri, ds_rootdn, ds_rootpw)
        manage_user_roles(multihost, "cn=nested", "unlock", "role")
        check_login_client(multihost, "foo1@example1", 'Secret123')
        check_login_client(multihost, "foo4@example1", 'Secret123')
        time.sleep(3)
        unlock_check(multihost, "foo1")
        unlock_check(multihost, "foo4")



""" Automation of localoverrides suite

:requirement: local_overrides
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
from __future__ import print_function
import pytest
from sssd.testlib.common.utils import sssdTools


@pytest.mark.usefixtures('setup_sssd', 'create_posix_usersgroups')
@pytest.mark.localoverrides
class TestLocalOverrides(object):
    """
    This is test case class for local overrides suite
    """
    @pytest.mark.tier1_2
    def test_0001_bz1919942(self, multihost,
                            backupsssdconf):
        """
        :title: ifp: sss_override does not take
         precedence over override_homedir directive
        :id: d2e98c70-26a0-11ec-bcf5-845cf3eff344
        :customerscenario: true
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1919942
        :steps:
          1. Edit sssd.conf and set homedir, i.e.
             `override_homedir = /home/%u1`
          2. Restart sssd and check homedir
          3. Use `sss_override` to set a
             different home directory
          4. Restart sssd and use getent command
             to check home directory, which is still set to same.
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
          4. Should succeed
        """
        tools = sssdTools(multihost.client[0])
        domain_name = tools.get_domain_section_name()
        client = sssdTools(multihost.client[0])
        domain_params = {'override_homedir': '/home/%u1'}
        client.sssd_conf(f'domain/{domain_name}', domain_params)
        multihost.client[0].service_sssd('restart')
        before = multihost.client[0].run_command("getent passwd "
                                                 "foo5@example1")
        assert before.returncode == 0
        multihost.client[0].run_command("sss_override "
                                        "user-add foo5@example1 -h "
                                        "/home/foo56")
        multihost.client[0].service_sssd('restart')
        after = multihost.client[0].run_command("getent passwd "
                                                "foo5@example1")
        assert "User:/home/foo56" in after.stdout_text



import re
from .utils import sssdTools


def find_logs(multihost, log_name, string_name):
    """This function will find strings in a log file
    log_name: Absolute path of log where the search will happen.
    string_name: String to search in the log file.
    """
    log_str = multihost.client[0].get_file_contents(log_name).decode('utf-8')
    assert string_name in log_str


def count_pattern_logs(multihost, log_name, string_name):
    """This function will find strings in a log file
    log_name: Absolute path of log where the search will happen.
    string_name: String to search in the log file.
    """
    return len(re.findall(string_name, multihost.client[0].get_file_contents(log_name).decode('utf-8')))


def client_backup_file(multihost, file_path):
    """This function will backup file in client machine
    file_path: String, Absolute path of file.
    """
    client = multihost.client[0]
    file_content = client.get_file_contents(file_path)
    client.put_file_contents(file_path+'_bkp', file_content)


def client_restore_file(multihost, file_path):
    """This function will restore file in client machine
    file_path: String, Absolute path of file.
    """
    client = multihost.client[0]
    file_content = client.get_file_contents(file_path)
    client.put_file_contents(file_path.split("_bkp")[0], file_content)


def client_remove_file(multihost, file_path):
    """This function will remove file in client machine
    file_path: String, Absolute path of file.
    """
    client = multihost.client[0]
    client.run_command(f"rm -vf {file_path}")


def count_lines(multihost, log_name):
    """This function will count no of lines of a file
    file_path: String, Absolute path of file.
    """
    return len(multihost.client[0].get_file_contents(log_name).decode('utf-8').split('\n')) - 1


def search_string_in_file(multihost, start_line, search_string, file_path):
    """This function will find strings in a log file
    file_path: String, Absolute path of file.
    search_string: String, to find in the log.
    start_line: int, number of line from where search will start
    """
    file = multihost.client[0].get_file_contents(file_path).decode('utf-8').split('\n')
    current_line = 1
    finding_list = []
    for line in file:
        if current_line >= start_line and search_string in line:
            finding_list.append(f"Found '{search_string}' in {file_path} at line {current_line}: {line.strip()}")
        current_line += 1
    return finding_list


def configure_proxy(multihost):
    """
        Configure local domain for the test.
    """
    tools = sssdTools(multihost.client[0])
    sssd_param = {'domains': 'local'}
    tools.sssd_conf('sssd', sssd_param)
    param = {'id_provider': 'proxy',
             'proxy_lib_name': 'files',
             'proxy_pam_target': 'sssd-shadowutils'}
    tools.sssd_conf('domain/local', param)
    tools.sssd_conf("sssd", {'services': 'nss, pam, ifp'}, action='update')



"""
IPA SSH Public Host Keys Tests.

:requirement: IPA: hostpublickeys

sss_ssh_knownhosts acquires SSH public keys for host and outputs them in OpenSSH known_hosts key format.
Support for 'KnownHostsCommand' and deprecate 'sss_ssh_knownhostsproxy'
"""

from __future__ import annotations

import time

import pytest
from sssd_test_framework.roles.client import Client
from sssd_test_framework.roles.ipa import IPA
from sssd_test_framework.topology import KnownTopology


@pytest.fixture(scope="module")
def public_keys(moduledatadir: str) -> list[str]:
    """
    Read list of public keys from module data file.

    :return: List of public keys.
    :rtype: list[str]
    """
    keys: list[str] = []
    with open(f"{moduledatadir}/public_keys") as f:
        for line in f.readlines():
            stripped = line.strip()
            if stripped:
                keys.append(stripped)

    return keys


@pytest.mark.ticket(gh=5518)
@pytest.mark.importance("low")
@pytest.mark.topology(KnownTopology.IPA)
def test_ipa__hostpublickeys_by_name(client: Client, ipa: IPA, public_keys: list[str]):
    """
    :title: sss_ssh_knownhosts returns public keys by name
    :setup:
        1. Create host with SSH key
        2. Configure SSSD with SSH responder
        3. Start SSSD
    :steps:
        1. Lookup SSH key
    :expectedresults:
        1. All public keys were printed
    :customerscenario: False
    """
    hostname = f"ssh-host.{ipa.domain}"
    ip = "10.255.251.10"

    ipa.host_account(hostname).add(ip=ip, sshpubkey=public_keys)
    client.sssd.enable_responder("ssh")
    client.sssd.start()

    result = client.sss_ssh_knownhosts(hostname)
    assert result.rc == 0, "Did not get OpenSSH known hosts public keys!"
    assert len(public_keys) == len(result.stdout_lines), "Did not get expected number of public keys!"
    for key in public_keys:
        assert f"{hostname} {key}" in result.stdout_lines, "Did not get expected public keys!"


@pytest.mark.ticket(gh=5518)
@pytest.mark.importance("low")
@pytest.mark.topology(KnownTopology.IPA)
def test_ipa__hostpublickeys_by_shortname(client: Client, ipa: IPA, public_keys: list[str]):
    """
    :title: sss_ssh_knownhosts returns public keys by short name using the search domain
    :setup:
        1. Create host with SSH key and add "search ipa.test" to /etc/resolv.conf
        2. Configure SSSD with SSH responder
        3. Start SSSD
    :steps:
        1. Lookup SSH key by running "sss_ssh_knownhosts ssh-host"
    :expectedresults:
        1. All public keys were printed
    :customerscenario: False
    """
    shortname = "ssh-host"
    hostname = f"{shortname}.{ipa.domain}"
    ip = "10.255.251.10"
    ipa.host_account(hostname).add(ip=ip, sshpubkey=public_keys)

    client.fs.append("/etc/resolv.conf", f"search {ipa.domain}")
    client.sssd.enable_responder("ssh")
    client.sssd.start()

    result = client.sss_ssh_knownhosts(shortname)
    assert result.rc == 0, "Did not get OpenSSH known hosts public keys!"
    assert len(public_keys) == len(result.stdout_lines), "Did not get expected number of public keys!"
    for key in public_keys:
        assert f"{shortname} {key}" in result.stdout_lines, "Did not get expected public keys!"


@pytest.mark.ticket(gh=5518)
@pytest.mark.importance("low")
@pytest.mark.topology(KnownTopology.IPA)
def test_ipa__hostpublickeys_by_ip(client: Client, ipa: IPA, public_keys: list[str]):
    """
    :title: sss_ssh_knownhosts returns public keys by IP
    :setup:
        1. Create host with SSH key
        2. Configure SSSD with SSH responder
        3. Start SSSD
    :steps:
        1. Lookup SSH key by running "sss_ssh_knownhosts $ip"
    :expectedresults:
        1. All public keys were printed
    :customerscenario: False
    """
    hostname = f"ssh-host.{ipa.domain}"
    ip = "10.255.251.10"
    ipa.host_account(hostname).add(ip=ip, sshpubkey=public_keys)

    client.sssd.enable_responder("ssh")
    client.sssd.start()

    result = client.sss_ssh_knownhosts(ip)
    assert result.rc == 0, "Did not get OpenSSH known hosts public keys!"
    assert len(public_keys) == len(result.stdout_lines), "Did not get expected number of public keys!"
    for key in public_keys:
        assert f"{ip} {key}" in result.stdout_lines, "Did not get expected public keys!"


@pytest.mark.ticket(bz=1926622)
@pytest.mark.integration
@pytest.mark.importance("low")
@pytest.mark.topology(KnownTopology.IPA)
def test_ipa__check_gssapi_authentication_indicator(client: Client, ipa: IPA):
    """
    :title: Check logs for authentication indicator
    :description:
        Checks that logs for the authentication indicator showing that the kerberos ticket was obtained using GSSAPI
    :setup:
        1. Configure SSSD for sudo and gssapi
        2. Start SSSD
        3. Create sudo configuration that allows user to run SUDO rules
    :steps:
        1. Login as the test user and obtain ticket
        2. Try 'sudo -l' as user
        3. Check if acquired service ticket has req. indicators: 0 in sssd_pam.log
        4. Update config with 'pam_gssapi_service, pam_gssaspi_indicator_map and restart sssd
        5. Login as the test user and obtain a new ticket
        6. Check if acquired service ticket has req. indicators: 2 in sssd_pam.log
    :expectedresults:
        1. Login successful and ticket obtained
        2. "Sudo -l" should run without password
        3. "indicators: 0" should be there in the sssd_pam.log
        4. Configuration is updated and SSSD is restarted
        5. Login successful and new ticket obtained
        6. "indicators: 2" should be there in the sssd_pam.log
    :customerscenario: True
    """
    user = ipa.user("user-1").add(password="Secret123")
    password = "Secret123"

    # In future some other string replacement module may be created, for now generic sed module is used.
    for path in ["/etc/pam.d/sudo", "/etc/pam.d/sudo-i"]:
        client.fs.sed(path=path, command="2s/^/auth sufficient pam_sss_gss.so debug\\n/", args=["-i"])

    client.sssd.config["pam"] = {
        "pam_gssapi_services": "sudo, sudo-i",
        "pam_gssapi_indicators_map": "hardened, sudo:pkinit, sudo-i:otp",
    }
    client.sssd.start()

    with client.ssh(user.name, password) as ssh:
        ssh.run(f"kinit {user.name}@{ipa.host.realm}", input=password)
        ssh.run("klist")
        ssh.disconnect()
    ipa.sudorule("testrule").add(user=user.name, host="ALL", command="sudo -l")
    assert not client.auth.sudo.list(user.name), "User found in sudo rule!"
    time.sleep(3)
    log1 = client.fs.read(client.sssd.logs.pam)
    assert "indicators: 0" in log1, "String `indicators: 0` not found in logs!"

    client.sssd.config["pam"] = {"pam_gssapi_services": "sudo, sudo-i", "pam_gssapi_indicators_map": "sudo-i:hardened"}
    client.sssd.clear(logs=True)
    client.sssd.restart()

    with client.ssh(user.name, password) as ssh:
        ssh.run(f"kinit {user.name}@{ipa.host.realm}", input=password)
        ssh.run("klist")
        ssh.disconnect()
    assert not client.auth.sudo.list(user.name), "User found in sudo rule!"
    time.sleep(3)
    log2 = client.fs.read(client.sssd.logs.pam)
    assert "indicators: 2" in log2, "String `indicators: 2` not found in logs!"



from sssd import util
from sssd.parser import SubparsersAction
from sssd import sss_analyze

class ErrorAnalyzer:
    """
    An error analyzer module, list if there is any error reported by sssd_be
    """
    module_parser = None
    print_opts = []

    def print_module_help(self, args):
        """
        Print the module parser help output

        Args:
            args (Namespace): argparse parsed arguments
        """
        self.module_parser.print_help()

    def setup_args(self, parser_grp, cli):
        """
        Setup module parser, subcommands, and options

        Args:
            parser_grp (argparse.Action): Parser group to nest
               module and subcommands under
        """
        desc = "Analyze error check module"
        self.module_parser = parser_grp.add_parser('error',
                                                   description=desc,
                                                   help='Error checker')

        subparser = self.module_parser.add_subparsers(title=None,
                                                      dest='subparser',
                                                      action=SubparsersAction,
                                                      metavar='COMMANDS')

        subcmd_grp = subparser.add_parser_group('Operation Modes')
        cli.add_subcommand(subcmd_grp, 'list', 'Print error messages found in backend',
                           self.print_error, self.print_opts)

        self.module_parser.set_defaults(func=self.print_module_help)

        return self.module_parser

    def print_error(self, args):
        err = 0
        utl = util.Utils()
        source = utl.load(args)
        component = source.Component.BE
        source.set_component(component, False)
        patterns = ['sdap_async_sys_connect request failed', 'terminated by own WATCHDOG',
            'ldap_sasl_interactive_bind_s failed', 'Communication with KDC timed out', 'SSSD is offline', 'Backend is offline',
            'tsig verify failure', 'ldap_install_tls failed', 's2n exop request failed']
        for line in utl.matched_line(source, patterns):
            err +=1
            print(line)
        if err > 0:
            print("For possible solutions please refer to https://sssd.io/troubleshooting/errors.html")
        return



#
#   Authors:
#       Pavel Brezina <pbrezina@redhat.com>
#
#   Copyright (C) 2017 Red Hat
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import argparse
from collections import OrderedDict
from sbus_Introspection import Introspectable
from sbus_Template import TemplateFile
from sbus_Generator import Generator
from sbus_DataType import DataType


class CodeGen:
    """
        Code generator support multiple introspection annotations that can
        modify the generator behavior per interface, method or property.

        * Annotations on methods:
        - codegen.CustomHandler
          - boolean, default is false
          - custom input and output handler
        - codegen.CustomInputHandler
          - boolean, default is false
          - handler parses its input parameters manually
        - codegen.CustomOutputHandler
          - boolean, default is false
          - handler parses its output parameters manually

        * Annotations on interfaces, methods or properties:
        - codegen.Name
          - string, default is not set
          - Name used to override member name when generating caller
        - codegen.Caller
          - boolean, default is true
          - Generate both synchronous and asynchronous callers
        - codegen.SyncCaller
          - boolean, default is true
          - Generate synchronous callers
        - codegen.AsyncCaller
          - boolean, default is true
          - Generate asynchronous callers
    """
    def __init__(self, options):
        # Temporarily change working directory so we can load the templates
        self.options = options
        self.templates = CodeGen.Templates(options)
        self.interfaces = OrderedDict()
        return

    def add(self, introspection_file):
        interfaces = Introspectable.Introspect(introspection_file)
        merged = self.interfaces

        for name, interface in interfaces.items():
            if name in self.interfaces:
                raise ValueError("Interface %s already exist!" % name)
            merged[name] = interface

        self.interfaces = OrderedDict(sorted(merged.items()))
        return

    def generate(self):
        Generator.GenerateCode(self.templates, self.interfaces)

    class Options:
        def __init__(self,
                     SbusHeadersPath,
                     UtilHeadersPath,
                     GeneratedHeadersPath,
                     WritePath,
                     FilePrefix,
                     SymbolPrefix,
                     IncludeHeaders):
            self.SbusHeadersPath = SbusHeadersPath
            self.UtilHeadersPath = UtilHeadersPath
            self.GeneratedHeadersPath = GeneratedHeadersPath
            self.WritePath = WritePath
            self.FilePrefix = FilePrefix
            self.SymbolPrefix = SymbolPrefix
            self.IncludeHeaders = []

            if IncludeHeaders is not None:
                self.IncludeHeaders = IncludeHeaders

            self.AbsolutePath = os.path.dirname(os.path.realpath(__file__))
            return

        def path(self, relative_path):
            return "%s/%s" % (self.AbsolutePath, relative_path)

    class Templates:
        GeneratedFiles = [
            "interface.h",
            "symbols.c",
            "symbols.h",
            "arguments.c",
            "arguments.h",
            "invokers.c",
            "invokers.h",
            "keygens.c",
            "keygens.h",
            "client_properties.h",
            "client_async.c",
            "client_async.h",
            "client_sync.c",
            "client_sync.h",
            "server.h"
        ]

        def __init__(self, options):
            self.files = {}
            for file in self.GeneratedFiles:
                self.files[file] = self.File(file, options)

        def get(self, name):
            return self.files[name].template

        def write(self):
            for file in self.files.values():
                file.write()

        class File:
            def __init__(self, name, options):
                self.options = options

                self.name = name
                self.outputFile = self.getAbsFilePath(name)

                self.template = TemplateFile(
                    options.path('./templates/' + name + '.tpl')
                )
                self.setHeader()

            def write(self):
                self.setFooter()
                self.template.write(self.outputFile, self.postprocess)

            def setHeader(self):
                if not self.template.has("file-header"):
                    return

                tpl = self.template.get("file-header")

                for header in self.options.IncludeHeaders:
                    tpl.add("custom-type-header",
                            {'custom-type-header': header})

                keys = {'sbus-path': self.options.SbusHeadersPath,
                        'util-path': self.options.UtilHeadersPath,
                        'file-guard': self.getHeaderGuard(self.name)}

                for file in CodeGen.Templates.GeneratedFiles:
                    if not file.endswith(".h"):
                        continue

                    name = file.replace(".h", "")
                    keys["header:" + name] = self.getRelFilePath(file)

                tpl.set(keys)

            def setFooter(self):
                if not self.template.has("file-footer"):
                    return

                keys = {'file-guard': self.getHeaderGuard(self.name)}

                self.template.get("file-footer").set(keys)

            def getHeaderGuard(self, name):
                guard = "_%s%s_" % (self.options.FilePrefix, name)
                return guard.replace('.', '_').upper()

            def getAbsFilePath(self, name):
                return "%s/%s%s" % \
                    (self.options.WritePath,
                     self.options.FilePrefix,
                     name)

            def getRelFilePath(self, name):
                return "%s/%s%s" % \
                    (self.options.GeneratedHeadersPath,
                     self.options.FilePrefix,
                     name)

            def postprocess(self, output):
                if self.options.SymbolPrefix is None:
                    return output

                return output.replace("_sbus_",
                                      "_sbus_%s_" % self.options.SymbolPrefix)


def InitializeDataTypes():
    """Define which D-Bus types are supported by code generator."""

    # Standard types
    DataType.Create("y", "uint8_t", '" PRIu8 "')
    DataType.Create("b", "bool", "d")
    DataType.Create("n", "int16_t", '" PRId16 "')
    DataType.Create("q", "uint16_t", '" PRIu16 "')
    DataType.Create("i", "int32_t", '" PRId32 "')
    DataType.Create("u", "uint32_t", '" PRIu32 "')
    DataType.Create("x", "int64_t", '" PRId64 "')
    DataType.Create("t", "uint64_t", '" PRIu64 "')
    DataType.Create("d", "double", "f")

    # String types
    DataType.Create("s", "const char *", "s", DBusType="s", RequireTalloc=True)
    DataType.Create("S", "char *", "s", DBusType="s", RequireTalloc=True)
    DataType.Create("o", "const char *", "s", DBusType="o", RequireTalloc=True)
    DataType.Create("O", "char *", "s", DBusType="o", RequireTalloc=True)

    # Array types
    DataType.Create("ay", "uint8_t *", RequireTalloc=True)
    DataType.Create("ab", "bool *", RequireTalloc=True)
    DataType.Create("an", "int16_t *", RequireTalloc=True)
    DataType.Create("aq", "uint16_t *", RequireTalloc=True)
    DataType.Create("ai", "int32_t *", RequireTalloc=True)
    DataType.Create("au", "uint32_t *", RequireTalloc=True)
    DataType.Create("ax", "int64_t *", RequireTalloc=True)
    DataType.Create("at", "uint64_t *", RequireTalloc=True)
    DataType.Create("ad", "double *", RequireTalloc=True)

    # String arrays
    DataType.Create("as", "const char **", DBusType="as", RequireTalloc=True)
    DataType.Create("aS", "char **", DBusType="as", RequireTalloc=True)
    DataType.Create("ao", "const char **", DBusType="ao", RequireTalloc=True)
    DataType.Create("aO", "char **", DBusType="ao", RequireTalloc=True)

    # Custom types
    DataType.Create("pam_data", "struct pam_data *",
                    DBusType="issssssuayuayiu", RequireTalloc=True)
    DataType.Create("pam_response", "struct pam_data *",
                    DBusType="uua(uay)", RequireTalloc=True)
    DataType.Create("ifp_extra", "hash_table_t *",
                    DBusType="a{sas}", RequireTalloc=True)


def main():
    InitializeDataTypes()

    parser = argparse.ArgumentParser(
        description='Generate sbus server and client code.'
    )

    parser.add_argument(
        'introspection', nargs='+',
        help="Path to introspection file"
    )
    required = parser.add_argument_group('Required arguments')
    required.add_argument(
        '--sbus', action="store", dest="sbuspath",
        help="Path to sbus header files as used in #include",
        required=True
    )
    required.add_argument(
        '--util', action="store", dest="utilpath",
        help="Path to util header files as used in #include",
        required=True
    )
    required.add_argument(
        '--headers', action="store", dest="headerpath",
        help="Path to generated header files as used in #include",
        required=True
    )
    required.add_argument(
        '--dest', action="store", dest="destpath",
        help="Path where the generated code will be stored",
        required=True
    )
    required.add_argument(
        '--fileprefix', action="store", dest="fileprefix",
        help="Name prefix for generated files",
        required=True
    )
    optional = parser.add_argument_group('Optional generator arguments')
    optional.add_argument(
        '--symbolprefix', action="store", dest="symbolprefix",
        help="Name prefix for generated global symbols",
        required=False
    )
    optional.add_argument(
        '-i', '--include', action='append', dest="include",
        help="Include header with definition of custom types",
        required=False
    )
    cmdline = parser.parse_args()

    opts = CodeGen.Options(
        SbusHeadersPath=cmdline.sbuspath,
        UtilHeadersPath=cmdline.utilpath,
        GeneratedHeadersPath=cmdline.headerpath,
        WritePath=cmdline.destpath,
        FilePrefix=cmdline.fileprefix,
        SymbolPrefix=cmdline.symbolprefix,
        IncludeHeaders=cmdline.include
    )

    codegen = CodeGen(opts)
    for file in cmdline.introspection:
        codegen.add(file)
    codegen.generate()


if __name__ == "__main__":
    main()



""" Automation of rfc2307bis

:requirement: rfc2307bis
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
import os
import re
import posixpath
import pytest
from sssd.testlib.common.utils import sssdTools
from constants import ds_instance_name

@pytest.mark.usefixtures('setup_sssd')
@pytest.mark.rfc2307bis
class Testrfc2307bis(object):
    """
    This is test case class for ldap rfc2307bis
        :setup:
          1. Configure SSSD to authenticate against directory server
          2. Enable debug_level to 9 in the 'nss', 'pam' and domain section
    """
    @staticmethod
    @pytest.mark.tier1_4
    def test_0001_tevent_loop_never_finished(multihost, backupsssdconf):
        """
        :title: rfc2307bis: tevent_loop_wait() never finishes
        :id: 253ecc50-5fd6-4326-9cb4-9ae68bd07067
        :description: If a signal handler is registered and unregistered
         later, tevent_loop_wait() never finishes.
        :bugzilla:
          https://bugzilla.redhat.com/show_bug.cgi?id=978962
          https://bugzilla.redhat.com/show_bug.cgi?id=994015
        :setup:
          tevent_loop.c is available in /tmp/ directory on client system
        :steps:
          1. Compile tevent_loop.c with gcc
          2. Run the binary file a.out, created from compiling tevent_loop.c, and
             capture the output of a.out in tevent_loop_output
          3. Assert 'We got through the loop' string in tevent_loop_output
        :expectedresults:
          1. Compilation should succeed
          2. The binary a.out should run successfully and output of it
             shouls be captured in tevent_loop_output
          3. The tevent_loop_output should contain 'We got through the loop'
          """
        tools = sssdTools(multihost.client[0])
        section = f'domain/{ds_instance_name}'
        params = {'ldap_schema': 'rfc2307bis'}
        tools.sssd_conf(section, params, action='update')
        tools.clear_sssd_cache()
        multihost.client[0].run_command('dnf --enablerepo=*-CRB install -y libtevent-devel', raiseonerr=False)
        cwd = os.path.dirname(os.path.abspath(__file__))
        source = posixpath.join(cwd, 'script/tevent_loop.c')
        multihost.client[0].transport.put_file(source, '/tmp/tevent_loop.c')
        multihost.client[0].run_command('gcc -ltalloc -ltevent /tmp/tevent_loop.c -o /tmp/a.out', raiseonerr=False)
        multihost.client[0].run_command('/tmp/a.out > /tmp/tevent_loop_output', raiseonerr=False)
        output = multihost.client[0].get_file_contents('/tmp/tevent_loop_output').decode('utf-8')
        log = re.compile(r'We got through the loop')
        assert log.search(output)



"""
Files Provider Test Cases

The files provider allows SSSD to use system users to authenticate.
This feature has been removed in SSSD 2.9.0 for the proxy provider.

:requirement: IDM-SSSD-REQ :: SSSD is default for local resolution
"""

from __future__ import annotations

import time

import pytest
from sssd_test_framework.roles.client import Client
from sssd_test_framework.topology import KnownTopology


@pytest.mark.importance("low")
@pytest.mark.builtwith("files-provider")
@pytest.mark.topology(KnownTopology.Client)
def test_files__root_user_is_ignored_on_lookups(client: Client):
    """
    :title: The root user is always ignored on sss service lookups
    :description: This ensures that the local root user is always returned
        and cannot be tampered with.
    :setup:
        1. Configure SSSD with files provider
        2. Start SSSD
    :steps:
        1. Lookup root user using sss service
        2. Lookup root user without the sss service
    :expectedresults:
        1. The root user is not found
        2. The root user is found
    :customerscenario: False
    """
    client.sssd.sssd["enable_files_domain"] = "true"
    client.sssd.start()

    assert client.tools.getent.passwd("root", service="sss") is None, "Root user is found using 'sss' service!"
    assert client.tools.getent.passwd("root"), "Root user is not found using all services!"


@pytest.mark.importance("low")
@pytest.mark.builtwith("files-provider")
@pytest.mark.topology(KnownTopology.Client)
def test_files__lookup_user(client: Client):
    """
    :title: Lookup user
    :setup:
        1. Create user
        2. Configure SSSD with files provider
        3. Start SSSD
    :steps:
        1. Lookup user
        2. Check results
    :expectedresults:
        1. User is found
        2. The uid matches
    :customerscenario: False
    """
    client.local.user("user1").add(uid=10001)
    client.sssd.sssd["enable_files_domain"] = "true"
    client.sssd.start()

    result = client.tools.getent.passwd("user1", service="sss")
    assert result is not None, "User not found!"
    assert result.uid == 10001, "UID does not match!"


@pytest.mark.importance("low")
@pytest.mark.builtwith("files-provider")
@pytest.mark.topology(KnownTopology.Client)
def test_files__enumeration_should_not_work(client: Client):
    """
    :title: Enumeration should not work
    :description: Enumeration pulls down the directory data and stores it locally.
        Running an unspecified getent will return all users or groups.
    :setup:
        1. Configure SSSD with files provider
        2. Start SSSD
    :steps:
        1. Run getent with nothing specified
    :expectedresults:
        1. Nothing found
    :customerscenario: False
    """
    client.sssd.sssd["enable_files_domain"] = "true"
    client.sssd.start()

    assert not client.host.conn.run("getent passwd -s sss").stdout, "Entries found!"


@pytest.mark.importance("low")
@pytest.mark.builtwith("files-provider")
@pytest.mark.topology(KnownTopology.Client)
def test_files__lookup_returns_the_latest_data(client: Client):
    """
    :title: Looking up a user returns the latest data
    :setup:
        1. Create user and specify home directory
        2. Configure SSSD with files provider
        3. Start SSSD
    :steps:
        1. Lookup user
        2. Check results
        3. Change user's home directory
        4. Lookup user again
        5. Check results
    :expectedresults:
        1. User is found
        2. The homedir matches
        3. Home directory is changed
        4. User is found
        5. Home directory reflects the new value
    :customerscenario: False
    """
    client.local.user("user1").add(password="Secret123", home="/home/user1-tmp")
    client.sssd.sssd["enable_files_domain"] = "true"
    client.sssd.start()

    result = client.tools.getent.passwd("user1", service="sss")
    assert result is not None, "User not found!"
    assert result.home == "/home/user1-tmp", "User's homedir is not correct!"

    client.local.user("user1").modify(home="/home/user1")

    time.sleep(1)
    result = client.tools.getent.passwd("user1", service="sss")
    assert result is not None, "User not found!"
    assert result.home == "/home/user1", "User's homedir is not correct!"



""" This module contains functions to do user lookup
and record the Memory consumption """

from subprocess import CalledProcessError
import time
import shlex
import csv
import numpy as np
from sssd.testlib.common.utils import PkiTools
from sssd.testlib.common.utils import sssdTools


class LookupPerf(object):
    """ Lookup Performance """
    def __init__(self, multihost=None, domain=None):
        self.multihost = multihost
        self.domain = domain

    def clear_cache_log(self, host):
        """ Clear sssd cache
        :param multihost object: host
        :Return: None
        """
        sssd_tools_inst = sssdTools(host)
        host.service_sssd('stop')
        sssd_log = "/var/log/sssd/sssd_%s.log" % (self.domain.lower())
        rm_log_cmd = "rm -f %s" % sssd_log
        host.run_command(rm_log_cmd)
        sssd_tools_inst.remove_sss_cache('/var/lib/sss/db')
        host.service_sssd('start')

    def zip_logs(self, host, log_file):
        """ gzip sssd domain logs
        :param multihost object: host
        :param str log_file: Log file name to be zipped
        :Return: None
        """
        gzip_cmd = 'gzip %s' % log_file
        host.run_command(gzip_cmd, raiseonerr=False)

    def user_lookup(self, host, runs, count, pattern):
        """ Perfor user lookup in a loop
        :param multihost object: host
        :param int runs: No of times the loop should run
        :param int count: No of users to be lookedup
        :param str pattern: The user name pattern
        :Return list ps_file_list: The list containing the file
        names where ps output was captured for sssd_be and sssd_nss process
        """
        ps_list = ['sssd_be', 'sssd_nss']
        ps_file_list = []
        clear_cache = 'sss_cache -E'
        sssd_log = "/var/log/sssd/sssd_%s.log" % (self.domain.lower())
        backup_sssd_log = "/var/log/sssd/sssd_%s_%d_%d.log" % (
            self.domain.lower(), runs, count)
        backup_log_cmd = "cp %s %s" % (sssd_log, backup_sssd_log)
        rpm_cmd = 'rpm -q --qf "%{name}-%{version}-%{release}" sssd'
        cmd = host.run_command(rpm_cmd)
        ver = cmd.stdout_text
        no_activity_count = 50
        self.clear_cache_log(host)
        for _ in range(runs):
            for num in range(1, count):
                id_cmd = 'id %s%d@%s' % (pattern, num, self.domain)
                try:
                    host.run_command(id_cmd)
                except CalledProcessError:
                    print("%s command failed" % id_cmd)
                for proc in ps_list:
                    ps_cmd1 = 'ps -eo pid,etime,pmem,pcpu,rss,vsize,args '\
                              '| grep %s | grep -v grep '\
                              '| grep -v implicit' % proc
                    cmd1 = host.run_command(ps_cmd1)
                    stat_file = 'psoutput-%s-%s-%d-%d.txt' % (ver, proc,
                                                              runs, count)
                    with open(stat_file, 'a+') as mon1:
                        mon1.write(cmd1.stdout_text.lstrip())
            host.run_command(clear_cache)
        # take memory usage when there is no activity
        for _ in range(no_activity_count):
            time.sleep(2)
            for proc in ps_list:
                ps_cmd2 = 'ps -eo pid,etime,pmem,pcpu,rss,vsize,args '\
                          '| grep %s | grep -v grep '\
                          '| grep -v implicit' % proc
                cmd2 = host.run_command(ps_cmd2)
                lookup_stats_file = 'psoutput-%s-%s-%d-%d.txt' % (ver, proc,
                                                                  runs, count)
                with open(lookup_stats_file, 'a+') as mon1:
                    mon1.write(cmd2.stdout_text.lstrip())
        # take backup of sssd domain log
        host.run_command(backup_log_cmd)
        # zip the log file
        self.zip_logs(host, backup_sssd_log)
        # remove the backup sssd domain log
        rm_log = 'rm -f %s' % backup_sssd_log
        host.run_command(rm_log, raiseonerr=False)
        for proc in ps_list:
            lookup_stats_file = 'psoutput-%s-%s-%d-%d.txt' % (ver, proc,
                                                              runs, count)
            ps_file_list.append(lookup_stats_file)
        return ps_file_list

    def get_vsz(self, stats_file):
        """ Compute Standard deviation of Virtual
        Memory size
        :param str stats_file: File name containing the psoutput command
        :Return list vsz: list containing Virtual Memory size
        """
        vsz = []
        with open(stats_file, 'r') as csvfile:
            monreader = csv.reader(csvfile, delimiter=' ')
            for row in monreader:
                try:
                    vsz.append(int(row[13]))
                except ValueError:
                    try:
                        vsz.append(int(row[10]))
                    except ValueError:
                        vsz.append(int(row[12]))
        return vsz

    def serial_lookup(self, host_list, runs, count, pattern):
        """ Do a lookup of users on list of hosts
        :param list host_list: List containing the multihost objects
        on which  id command should be run
        :param int runs: No of times the loop should run
        :param int count: No of users to be lookedup
        :param str pattern: The user name pattern
        :Return list stats_file_list: List containing ps output
        file names
        """
        stats_file_list = []
        for host in host_list:
            stats_file_list.append(self.user_lookup(host, runs,
                                                    count, pattern))
        return stats_file_list

    def std_deviation(self, vsz_list):
        """ Get Standard deviation
        :param list vsz_list: List containing vsz from psoutput
        :Return int: Standard deviation of the list
        """
        return np.std(vsz_list, ddof=1)

    def create_plotfile(self, host_list, stats_file_list, runs, count, proc):
        """ Create GNU Plot file from a
        given file containing ps memory output
        :param list host_list: List contaning multihost objects
        :param list stats_file_list: List containing 2 ps output files
         which needs to be used for plotting.
        :param int runs: No of times the user lookup was run
        :param int count: No of users looked up
        :param str proc: SSSD Process Name
        :Return str plot_file: GNU Plot file
        """
        list_max = []
        list_min = []
        plot_range = []
        ver = []
        for stat_file in stats_file_list:
            vsz_list = self.get_vsz(stat_file)
            list_max.append(max(vsz_list))
            list_min.append(min(vsz_list))
        plot_range.append(int(min(list_min)))
        plot_range.append(int(max(list_max)))
        if 'nss' in proc:
            ytics = abs((plot_range[0] - plot_range[1]) * 15 / 100)
        else:
            ytics = abs((plot_range[0] - plot_range[1]) * 5 / 100)
        rpm_cmd = 'rpm -q --qf "%{name}-%{version}-%{release}" sssd'
        for host in host_list:
            cmd = host.run_command(rpm_cmd)
            ver.append(cmd.stdout_text)
        title = 'Memory consumption(vsz) of %s users for %d ' \
                ' user(%d runs)' % (proc, count, runs)
        plot_file = 'perf_gnuplot_%s_%d_%d.gnuplot' % (proc, runs, count)
        output_png = 'mem_graph_%s_%d_%d.png' % (proc, runs, count)
        with open(plot_file, 'w+') as plotfile:
            plotfile.write('set term png small size 800,600\n')
            plotfile.write('set title "%s\n' % title)
            plotfile.write('set output "%s"\n' % output_png)
            plotfile.write('set ylabel "VSZ"\n')
            plotfile.write('set key right bottom\n')
            plotfile.write('set ytics %d\n' % ytics)
            plotfile.write('set yrange [%s:%s]\n' % (plot_range[0] - 1000,
                                                     plot_range[1] + 3000))
            plotfile.write('plot "%s" using 6 with lines axes '
                           'x1y1 title "%s" lt rgb "red", \\\n'
                           % (stats_file_list[1], ver[1]))
            plotfile.write('     "%s" using 6 with lines axes '
                           'x1y1 title "%s" lt rgb "green" \n'
                           % (stats_file_list[0], ver[0]))
            return plot_file

    def run_gnuplot(self, plot_file):
        """ Run gnuplot
        :param str plot_file: Name of the plotfile
        :Return: None
        """
        pki_inst = PkiTools()
        gnuplot_cmd = 'gnuplot %s' % plot_file
        pki_inst.execute(shlex.split(gnuplot_cmd))



# Authors: Simo Sorce <ssorce@redhat.com>
#          Alexander Bokovoy <abokovoy@redhat.com>
#          Tomas Babej <tbabej@redhat.com>
#
# Copyright (C) 2007-2014  Red Hat
# see file 'COPYING' for use and warranty information
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" Authconfig Module """
from __future__ import print_function
from subprocess import CalledProcessError
import os

FILES_TO_NOT_BACKUP = ['passwd', 'group', 'shadow', 'gshadow']


class RedHatAuthConfig(object):
    """
    AuthConfig class implements a system-independent interface to configure
    system authentication resources. In Red Hat systems this is done with
    authconfig(8) utility.

    AuthConfig class is nothing more than a tool to gather configuration
    options and execute their processing. These options are then converted by
    an actual implementation to a series of system calls to appropriate
    utilities performing real configuration.

    If you need to re-use existing AuthConfig instance for multiple runs,
    make sure to call 'AuthConfig.reset()' between the runs.
    """

    def __init__(self, host):
        """ Initialize host
        :param str host: hostname
        """
        self.host = host
        self.parameters = {}

    def enable(self, option):
        """
        Option to be passed to authconfig
        :param str option: authconfig options
        """
        self.parameters[option] = True
        return self

    def disable(self, option):
        """
        Disable options
        :param str option: authconfig options
        """
        self.parameters[option] = False
        return self

    def add_option(self, option):
        """
        Add option
        :param str option: authconfig options
        """
        self.parameters[option] = None
        return self

    def add_parameter(self, option, value):
        """
        Add parameters
        :param str option: authconfig options
        :param str values: values
        """
        self.parameters[option] = [value]
        return self

    def reset(self):
        """
        Reset to the default
        """
        self.parameters = {}
        return self

    def build_args(self):
        """
        Build argument list from options provided
        """
        args = []
        print("parameters passed: ", self.parameters)
        for (option, value) in self.parameters.items():
            if type(value) is bool:
                if value:
                    args.append("--enable%s" % (option))
                else:
                    args.append("--disable%s" % (option))
            elif type(value) in (tuple, list):
                args.append("--%s" % (option))
                args.append("%s" % (value[0]))
            elif value is None:
                args.append("--%s" % (option))
            else:
                args.append("--%s%s" % (option, value))
        return args

    def execute(self, update=True):
        """ Execute authconfig command """
        if update:
            self.add_option("update")

        args = self.build_args()
        auth_cmd = ['/usr/sbin/authconfig'] + args
        cmd = self.host.run_command(auth_cmd, set_env=False, raiseonerr=False)
        if cmd.returncode != 0:
            raise Exception("Failed to run Authconfig")

    def backup(self, path):
        """ Backup existing authconfig options
        :param str path: path where existing files are backed
        """
        cmd = self.host.run_command(['/usr/sbin/authconfig', '--savebackup',
                                     path], set_env=False, raiseonerr=False)
        if cmd.returncode != 0:
            raise Exception("Unable to save backup")
        # do not backup these files since we don't want to mess with
        # users/groups during restore. Authconfig doesn't seem to mind about
        # having them deleted from backup dir
        files_to_remove = [os.path.join(path, f) for f in FILES_TO_NOT_BACKUP]
        for filename in files_to_remove:
            try:
                self.host.run_command(['rm', '-f', filename], set_env=False)
            except CalledProcessError:
                pass

    def restore(self, path):
        """ Restore from backup
        :param str path: backup path
        """
        self.host.run_command(["/usr/sbin/authconfig", "--restorebackup",
                               path], set_env=False, raiseonerr=False)



""" Automation of sanity/services suite

:requirement: IDM-SSSD-REQ : Configuration and Service Management
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
from __future__ import print_function
import subprocess
import re
import pytest
import random
import string
from sssd.testlib.common.utils import sssdTools


def exceute_cmd(multihost, command):
    cmd = multihost.client[0].run_command(command)
    return cmd


@pytest.mark.usefixtures('setup_sssd')
@pytest.mark.services
class TestServices(object):
    """ SSSD sanity services """
    @pytest.mark.tier1
    def test_0001_bz1432010(self, multihost):
        """
        :title: IDM-SSSD-TC: sanity: services: SSSD ships a drop-in
         configuration snippet in /etc/systemd/system
        :id: 755973b3-200c-456d-8057-f4a6b093e3f4
        """
        rpm_grep = 'rpm -ql sssd-common'
        cmd = multihost.client[0].run_command(rpm_grep, raiseonerr=False)
        assert cmd.stdout_text.find('/etc/systemd/system') == -1

    @pytest.mark.tier1
    def test_0002_1736796(self, multihost, localusers):
        """
        :title: config: "default_domain_suffix" should not
         cause files domain entries to be qualified, this can break sudo access
        :id: 4b7bdeff-51ba-46ed-b8e1-0685515b87a0
        """
        users = localusers
        for user in users.keys():
            allow_sudo = '%s    ALL=(ALL) NOPASSWD:ALL' % user
            sudoers_file = '/etc/sudoers.d/%s' % user
            multihost.client[0].put_file_contents(sudoers_file, allow_sudo)
        tools = sssdTools(multihost.client[0])
        sssd_params = {'default_domain_suffix': 'foo',
                       'domains': 'LOCAL'}
        tools.sssd_conf('sssd', sssd_params)
        domain_section = 'domain/LOCAL'
        domain_params = {'id_provider': 'proxy',
                        'proxy_lib_name': 'files',
                        'proxy_pam_target': 'sssd-shadowutils'}
        tools.sssd_conf(domain_section, domain_params)
        multihost.client[0].service_sssd('restart')
        failures = []
        for user in users.keys():
            cmd = multihost.client[0].run_command(
                f'su - {user} -c "id"', raiseonerr=False)
            if f'{user}@implicit_files' in cmd.stdout_text:
                failures.append(
                    f"id command contains implicit_files for {user}")
            cmd2 = multihost.client[0].run_command(
                f'su - {user} -c "sudo su - -c id"', raiseonerr=False)
            if cmd2.returncode != 0:
                journalctl_cmd = 'journalctl -x -n 100 --no-pager'
                multihost.client[0].run_command(journalctl_cmd)
                failures.append(f"'sudo su - -c id' cmd failed for {user}")
        for user in users.keys():
            sudoers_file = '/etc/sudoers.d/%s' % user
            delete_file = 'rm -f %s' % sudoers_file
            multihost.client[0].run_command(delete_file)
        assert not failures, "\n".join(failures)

    @pytest.mark.tier1
    def test_0003_bz1713368(self, multihost):
        """
        :title: services: Add sssd-dbus package as a dependency of sssd-tools
        :id: dfbdcb9f-09ed-4467-97a6-0407c779dd08
        :customerscenario: True
        """
        # sssd-dbus is a weak dependency of sssd-tools package so we did not
        # get sssd-dbus in '# yum deplist sssd-tools' command
        version = float(re.findall(r"\d+\.\d+", multihost.client[0].distro)[0])
        if version >= 9:
            cmd = multihost.client[0].run_command('yum repoquery'
                                                  ' --requires sssd-tools',
                                                  raiseonerr=False)
        else:
            cmd = multihost.client[0].run_command('yum repoquery --recommends'
                                                  ' sssd-tools',
                                                  raiseonerr=False)
        if cmd.returncode == 0:
            find = re.compile(r'sssd-dbus')
            result = find.search(cmd.stdout_text)
            assert result is not None

    @pytest.mark.tier1
    def test_0004_membership_with_files_provider(self, multihost):
        """
        :title: services: SSSD must be able to resolve
         membership involving root with files provider
        :id: ca7adb25-6a2b-4f09-b5ed-dc083226c6c9
        :customerscenario: True
        :bugzilla:
         https://bugzilla.redhat.com/show_bug.cgi?id=1794607
        """
        # take backup:
        back_up = "cp -vf /etc/pam.d/su /etc/pam.d/su_bkp"
        cmd = multihost.client[0].run_command(back_up, raiseonerr=False)
        ps_cmd = "sed -i '/sufficient/s/#auth/auth/' /etc/pam.d/su"
        cmd = multihost.client[0].run_command(ps_cmd)
        useradd = "useradd -g root -G wheel user_its_some"
        cmd = multihost.client[0].run_command(useradd, raiseonerr=False)
        check_wheel = "runuser -l user_its_some -c 'getent group wheel'"
        cmd = multihost.client[0].run_command(check_wheel, raiseonerr=False)
        if 'wheel:x:10:user_its_some' in cmd.stdout_text:
            status = 'PASS'
        else:
            status = 'FAIL'
        # restore
        restore = "cp -vf /etc/pam.d/su_bkp /etc/pam.d/su "
        cmd = multihost.client[0].run_command(restore, raiseonerr=False)
        # clean up
        deluser = "userdel  user_its_some"
        cmd = multihost.client[0].run_command(deluser, raiseonerr=False)
        if cmd.returncode != 0:
            status = 'FAIL'
        assert status == 'PASS'

    @pytest.mark.tier1
    def test_0005_sssd_stops_monitoring(self, multihost):
        """
        :title: services: When the passwd or group files
         are replaced, sssd stops monitoring the file for
         inotify events, and no updates are triggered
        :id: c6bf72fa-75be-4004-b04b-b5ea3e662c7d
        :customerscenario: True
        :bugzilla:
         https://bugzilla.redhat.com/show_bug.cgi?id=1827432
        """
        group_bkp = "cp -vf /etc/group /etc/group_bkp"
        multihost.client[0].run_command(group_bkp, raiseonerr=False)
        group_tmp = "echo g10001:x:10001: >> /etc/group.tmp"
        multihost.client[0].run_command(group_tmp, raiseonerr=False)
        group_move = "mv -f /etc/group.tmp /etc/group"
        multihost.client[0].run_command(group_move, raiseonerr=False)
        group_add = "echo g10002:x:10002: >> /etc/group"
        multihost.client[0].run_command(group_add, raiseonerr=False)
        cmd_getent = "getent group g10002"
        cmd = multihost.client[0].run_command(cmd_getent, raiseonerr=False)
        if "g10002:x:10002" in cmd.stdout_text:
            status = "PASS"
        else:
            status = "FALSE"
        group_restore = "cp -vf /etc/group_bkp /etc/group"
        multihost.client[0].run_command(group_restore, raiseonerr=False)
        assert status == "PASS"

    @pytest.mark.tier1_2
    def test_0006_bz1909755(self, multihost, backupsssdconf):
        """
        :title: Suppress log message "[sssd] [service_signal_done]
         (0x0010): Unable to signal service [2]:
         No such file or directory" during logrote
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1909755
        :customerscenario: true
        :id: a4f5d404-070b-11ec-8055-845cf3eff344
        :steps:
          1. Find main sssd process id
          2. Send SIGHUP
          3. There should not be any logs for
             'Unable to signal service .* No such
             file or directory
             modifyTimestamp' in the filter
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
        """
        multihost.client[0].service_sssd('restart')
        proces_id = int(exceute_cmd(multihost,
                                    "pidof sssd").stdout_text.split()[0])
        exceute_cmd(multihost, f"kill -1 {proces_id}")
        with pytest.raises(subprocess.CalledProcessError):
            exceute_cmd(multihost, 'grep -ri "Unable '
                                   'to signal service .* No '
                                   'such file or directory" '
                                   '/var/log/sssd')

    @pytest.mark.tier1_2
    def test_0007_bz971435(self, multihost, backupsssdconf):
        """
        :title: Enhance sssd init script so that it would source a configuration
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=971435
        :id: ba764db9-90e0-4e6a-bd3c-b7bd6221f340
        :steps:
          1. Add 'PIZZA=YUMMY' string in /etc/sysconfig/sssd
          2. Restart the sssd
          3. Check pid of sssd and grep the string under /proc directory
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should succeed
        """
        # port bash to pytest
        tools = sssdTools(multihost.client[0])
        sssd_params = {'debug_level': '9'}
        tools.sssd_conf('sssd', sssd_params)
        multihost.client[0].run_command('echo "PIZZA=YUMMY" > /etc/sysconfig/sssd', raiseonerr=False)
        multihost.client[0].service_sssd('restart')
        process_id = (exceute_cmd(multihost, "pidof sssd").stdout_text.split()[0])
        file_for_grep = f"/proc/{process_id}/environ"
        grep_cmd = f'grep "PIZZA=YUMMY" {file_for_grep}'
        cmd_check = multihost.client[0].run_command(grep_cmd, raiseonerr=False)
        multihost.client[0].run_command('rm -f /etc/sysconfig/sssd', raiseonerr=False)
        assert cmd_check.returncode == 0, "string 'PIZZA=YUMMY' not found in /proc file"

    @pytest.mark.tier1_2
    def test_0008_bz1516266(self, multihost, backupsssdconf):
        """
        :title: detailed debug and system-log message if krb5_init_context failed
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1516266
        :id: fe126445-21d2-4572-b32a-b66c8486569f
        :steps:
          1. Update the sssd.conf with id and auth provider
          2. Add 'includedir /var/lib/sss/pubconf/krb5.include.d/' in krb5.conf
          3. Check sssd status, sssd should start
          4. Revert back the krb.conf and add 'includedir /var/lib/sss/pubconf/krb5.include.d/$$$$' in krb5.conf
             here $$$ is spaces after string
          5. Check sssd status, sssd should fail to start
        :expectedresults:
          1. Successfully upadated sssd.conf
          2. Successfully added string in krb5.conf
          3. Successfully start the sssd service
          4. Successfully reverted the krb5.conf and added string with spaces at the end
          5. SSSD failing to start
        """
        # port bash to pytest
        tools = sssdTools(multihost.client[0])
        sssd_params = {'domains': 'LDAP',
                       'debug_level': '9'}
        tools.sssd_conf('sssd', sssd_params)
        domain_section = 'domain/LDAP'
        domain_params = {'id_provider': 'ldap',
                         'ldap_uri': 'ldap://ldap.example.com',
                         'ldap_search_base': 'dc=example,dc=com',
                         'auth_provider': 'krb5',
                         'krb5_server': 'kerberos.example.com',
                         'krb5_realm': 'EXAMPLE.COM',
                         'debug_level': '9'}
        tools.sssd_conf(domain_section, domain_params)
        # stop sssd, delete logs and cache, start sssd
        tools.clear_sssd_cache()
        random_file1 = 'random' + ''.join(
            random.choice(string.ascii_lowercase) for i in range(5))
        take_bk = 'cp -f /etc/krb5.conf /etc/krb5.conf.backup'
        multihost.client[0].run_command(take_bk, raiseonerr=False)
        cmd_to_add = '{ echo "includedir /var/lib/sss/pubconf/krb5.include.d' \
                     '/"; cat /etc/krb5.conf; } > ' \
                     f'/tmp/{random_file1}'
        multihost.client[0].run_command(cmd_to_add, raiseonerr=False)
        copy_radom = f'mv -f /tmp/{random_file1} /etc/krb5.conf'
        multihost.client[0].run_command(copy_radom, raiseonerr=False)
        start_sssd = multihost.client[0].service_sssd('restart')
        restore_krb = 'cp -f /etc/krb5.conf.backup /etc/krb5.conf'
        multihost.client[0].run_command(restore_krb, raiseonerr=False)
        assert start_sssd == 0, "SSSD service fails to start after " \
                                "adding string in /etc/krb5.conf"
        random_file2 = 'random' + ''.join(
            random.choice(string.ascii_lowercase) for i in range(5))
        # Dont remove the extra space in bellow line. Its needed for this test
        cmd_with_sp = '{ echo "includedir /var/lib/sss/pubconf/krb5.include' \
                      '.d/  "; cat /etc/krb5.conf; } > ' \
                      f'/tmp/{random_file2}'
        multihost.client[0].run_command(cmd_with_sp, raiseonerr=False)
        copy_radom2 = f'mv -f /tmp/{random_file2} /etc/krb5.conf'
        multihost.client[0].run_command(copy_radom2, raiseonerr=False)
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/log/sssd')
        # Here sssd expected to fail, and if we use function from common
        # to start, it will raise an exception, so added command to check
        # log message after sssd fails to start.
        start_sssd_sp = multihost.client[0].run_command(
            'systemctl start sssd', raiseonerr=False)
        log_str = multihost.client[0].get_file_contents(
            '/var/log/sssd/sssd_LDAP.log').decode('utf-8')
        multihost.client[0].run_command(restore_krb, raiseonerr=False)
        multihost.client[0].run_command(
            'rm -f /etc/krb5.conf.backup', raiseonerr=False)
        assert start_sssd_sp.returncode != 0
        assert re.compile(r'Failed to init Kerberos context .Included '
                          r'profile directory could not be read').search(
            log_str)



"""Automation tests for dynamic DNS

:requirement: dyndns
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
+:status: approved
"""
# flake8: noqa: W605

import pytest
import time
import random
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.utils import ADDNS


@pytest.fixture(scope="class")
def change_client_hostname(session_multihost, request):
    """ Change client hostname to client[XX].domain.com """
    cmd = session_multihost.client[0].run_command('hostname', raiseonerr=False)
    old_hostname = cmd.stdout_text.rstrip()
    ad_domain = session_multihost.ad[0].domainname
    new_hostname = "client" + str(random.randrange(99)) + "." + ad_domain
    session_multihost.client[0].run_command(f'hostnamectl set-hostname \"{new_hostname}\"')

    def restore_hostname():
        """ Restore hostname """
        session_multihost.client[0].run_command(f'hostnamectl set-hostname {old_hostname}')

    request.addfinalizer(restore_hostname)


@pytest.fixture(scope="class")
def disable_dns_forwarders(session_multihost, request):
    """ Disables recursive lookups on DNS server """
    session_multihost.ad[0].run_command('dnscmd.exe /config /norecursion 1', raiseonerr=False)

    def enable_dns_forwarders():
        """ Enables recursion lookups on DNS servers """
        session_multihost.ad[0].run_command('dnscmd.exe /config /norecursion 0', raiseonerr=False)

    request.addfinalizer(enable_dns_forwarders)


@pytest.fixture(scope="class")
def reverse_zone(session_multihost, request):
    """ Creates reverse zones """
    dns = ADDNS(session_multihost.ad[0])
    ip = session_multihost.ad[0].ip.split(".")
    ip1 = session_multihost.client[0].ip.split(".")
    network = str(f'{ip[2]}.{ip[1]}.{ip[0]}.in-addr.arpa')
    dns.add_zone(network)
    if ip[2] != ip1[2]:
        network1 = str(f'{ip1[2]}.{ip1[1]}.{ip1[0]}.in-addr.arpa')
        assert dns.add_zone(network1)

    def remove_reverse_zone():
        """ Delete reverse zones """
        dns.del_zone(network)
        if ip[2] != ip1[2]:
            dns.del_zone(network1)

    request.addfinalizer(remove_reverse_zone)


@pytest.fixture(scope="function")
def extra_network(session_multihost, request):
    """ Create reverse zone """
    dns = ADDNS(session_multihost.ad[0])
    network = "1.168.192.in-addr.arpa"
    assert dns.add_zone(network)

    def remove_extra_network():
        """ Delete reverse zone"""
        dns.del_zone(network)

    request.addfinalizer(remove_extra_network)


@pytest.fixture(scope="function")
def extra_interface(session_multihost, request):
    """ Create extra interface """
    extra_interface = 'ibm' + str(random.randrange(99))
    extra_ip = '192.168.1.' + str(random.randrange(2, 255))
    session_multihost.client[0].run_command('ip link add ' + extra_interface + ' type dummy')
    session_multihost.client[0].run_command('ip addr add ' + extra_ip + ' dev ' + extra_interface)

    def remove_interface():
        """ Delete interface """
        session_multihost.client[0].run_command('ip addr flush dev ' + extra_interface)
        session_multihost.client[0].run_command('ip link del ' + extra_interface)

    request.addfinalizer(remove_interface)
    return extra_interface, extra_ip


@pytest.mark.usefixtures("reverse_zone", "disable_dns_forwarders", "change_client_hostname")
@pytest.mark.dyndns
@pytest.mark.tier2
class TestDynDns(object):

    @staticmethod
    @pytest.mark.c_ares
    def test_0001_verify_with_default_setting(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: verify with default settings
        :id: 183cc040-0b8c-47af-8b19-8bcb72c3f001
        :steps:
          1. Join client to AD domain
          2. Perform DNS lookup for hostname
          3. Perform DNS lookup for IP
        :expectedresults:
          1. Client joins domain
          2. Hostname resolves correctly
          3. IP resolves correctly
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        ip = multihost.client[0].ip

        client.clear_sssd_cache()
        # Update function with IPV6 support is added to output what failed
        assert dns.find_a(hostname, ip)
        assert dns.find_ptr(hostname, ip)

    @staticmethod
    @pytest.mark.c_ares
    def test_0002_verify_when_dyndns_update_set_to_false(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: verify when dyndns update set to false
        :id: 0cdbd0fe-4c67-4192-b44b-04e4227b5858
        :steps:
          1. Join client to AD domain and delete all client DNS records
          2. Perform DNS lookup for hostname
          3. Perform DNS lookup for IP
        :expectedresults:
          1. Client joins domain
          2. Hostname resolves incorrectly
          3. IP resolves incorrectly
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        domain = multihost.ad[0].domainname
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        ip = str(multihost.client[0].ip)

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_update': 'false'}
        client.sssd_conf(domain_section, sssd_params)
        dns.del_record(hostname)
        dns.del_record(ip)
        client.clear_sssd_cache()
        assert dns.find_a(hostname, ip) is not True
        assert dns.find_ptr(hostname, ip) is not True

    @staticmethod
    @pytest.mark.c_ares
    def test_0003_verify_with_dyndns_ttl_functionality(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: verify with dyndns ttl functionality
        :id: ffb8ee9b-2a82-4917-82b0-d87fd066a24c
        :steps:
          1. Join client to AD domain and delete all client DNS records
          2. Configure sssd.conf with dyndns_ttl = 9200
          3. Perform DNS lookup for hostname
          4. Perform DNS lookup for IP
        :expectedresults:
          1. Client joins domain
          2. Should succeed
          3. Hostname resolves with 9200 ttl
          4. IP resolves with 9200 ttl
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        domain = multihost.ad[0].domainname
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        ip = multihost.client[0].ip

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_update': 'true', 'dyndns_ttl': '9200'}
        client.sssd_conf(domain_section, sssd_params)
        dns.del_record(hostname)
        dns.del_record(ip)
        client.clear_sssd_cache()

        assert dns.find_a(hostname, ip)
        assert dns.find_ptr(hostname, ip)
        assert '9200' in dns.print_zone(domain)

    @staticmethod
    def test_0004_check_dyndns_iface_with_existing_interfaces(
            multihost, adjoin, extra_network, extra_interface):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: check dyndns iface with existing interface
        :id: 166ee0a0-6f19-4be0-b5db-7373dde82862
        :steps:
          1. Join client to AD domain and delete all client DNS records
          2. Create extra interface
          3. Perform DNS lookup for hostname on extra interface
          4. Perform DNS lookup for IP on extra interface
        :expectedresults:
          1. Client joins domain
          2. Should succeed
          3. Hostname resolves correctly on extra interface
          4. IP resolves correctly on extra interface
        """
        adjoin(membersw='adcli')
        (extra_interface, extra_ip) = extra_interface
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        domain = multihost.ad[0].domainname
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        ip = multihost.client[0].ip

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_iface': extra_interface}
        client.sssd_conf(domain_section, sssd_params)
        dns.del_record(hostname)
        dns.del_record(ip)
        dns.del_record(extra_ip)
        client.clear_sssd_cache()

        assert dns.find_a(hostname, extra_ip)
        assert dns.find_ptr(hostname, extra_ip)
        assert not dns.find_ptr(hostname, ip)

    @staticmethod
    @pytest.mark.c_ares
    def test_0005_check_dyndns_iface_with_non_existing_interfaces(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: check dyndns iface with non-existing interfaces
        :id: 412f684a-5a68-472d-86b1-93019bb53e6f
        :steps:
          1. Join client to AD domain and delete all client DNS records
          2. Perform DNS lookup for hostname on bogus interface
          3. Perform DNS lookup for IP on bogus interface
        :expectedresults:
          1. Client joins domain
          2. Hostname does not resolve correctly on bogus interface
          3. IP does not resolve correctly on bogus interface
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        domain = multihost.ad[0].domainname
        ip = multihost.client[0].ip
        dns.del_record(hostname)
        dns.del_record(ip)

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_iface': 'non_existent'}
        client.sssd_conf(domain_section, sssd_params)
        client.clear_sssd_cache()

        assert dns.find_a(hostname, ip) is not True
        assert dns.find_ptr(hostname, ip) is not True

    @staticmethod
    @pytest.mark.c_ares
    def test_0006_check_with_dyndns_refresh_interval(multihost, adjoin, extra_network, extra_interface):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: check with dyndns refresh interval
        :id: 54765188-b21f-4ba1-890b-953eec9b696d
        :steps:
          1. Join client to AD domain and delete all client DNS records
          2. Create extra interface and assign IP
          3. Set refresh interval and dyndns iface in sssd.conf
          4. Perform DNS lookup for hostname on extra interface
          5. Perform DNS lookup for IP on extra interface
          6. Change IP on extra interface
          7. Let SSSD update DNS
          8. Perform DNS lookup for hostname on extra interface
          9. Perform DNS lookup for IP on extra interface
        :expectedresults:
          1. Client joins domain
          2. Should succeed
          3. Should succeed
          4. Hostname resolves correctly on extra interface
          5. IP resolves correctly on extra interface
          6. New IP address is assigned for extra interface
          7. SSSD updates DNS
          8. Hostname resolves correctly on extra interface with new IP
          9. IP resolves correctly on extra interface with new hostname
        """
        (extra_interface, extra_ip) = extra_interface
        extra_ip_after_refresh = '192.168.1.' + str(random.randrange(2, 255))
        while extra_ip == extra_ip_after_refresh:
            extra_ip_after_refresh = '192.168.1.' + str(random.randrange(2, 255))
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        domain = multihost.ad[0].domainname
        ip = multihost.client[0].ip
        dns.del_record(hostname)
        dns.del_record(ip)

        adjoin(membersw='adcli')
        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_refresh_interval': '81', 'dyndns_iface': extra_interface}
        client.sssd_conf(domain_section, sssd_params)
        client.clear_sssd_cache()
        assert dns.find_a(hostname, extra_ip)
        assert dns.find_ptr(hostname, extra_ip)

        multihost.client[0].run_command('ip addr flush dev ' + extra_interface)
        multihost.client[0].run_command('ip addr change ' + extra_ip_after_refresh + ' dev ' + extra_interface)
        time.sleep(83)

        assert dns.find_a(hostname, extra_ip_after_refresh)
        assert dns.find_ptr(hostname, extra_ip_after_refresh)

    @staticmethod
    @pytest.mark.c_ares
    def test_0007_set_dyndns_update_ptr_false_ptr_records_are_absent(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: set dyndns update ptr false ptr records are absent
        Will test parameter dyndns_update_ptr = false when no records exists
        NOTE: dyndns_refresh_interval lowest value is 60 seconds, with a 21 second timeout so the value is 81
        :id: e7bac85a-0504-4487-b26e-129a99121810
        :steps:
        1. Join client to AD domain and delete all client DNS records
        2. Delete both A and PTR records
        3. Set dyndns_update_ptr = false in sssd.conf
        4. Start SSSD
        5. Use DNS to check the host and IP
        :expectedresults:
        1. Client joins domain
        2. Host records are removed from the DNS server
        3. SSSD is configured
        4. SSSD is started
        5. Host record can be resolved and the PTR record cannot
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname_cmd = multihost.client[0].run_command('hostname')
        hostname = hostname_cmd.stdout_text.rstrip()
        domain = multihost.ad[0].domainname
        ip = multihost.client[0].ip

        dns.del_record(hostname)
        dns.del_record(ip)
        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_refresh_interval': '81', 'dyndns_update_ptr': 'false'}
        client.sssd_conf(domain_section, sssd_params)
        client.clear_sssd_cache()

        assert dns.find_a(hostname, ip)
        assert dns.find_ptr(hostname, ip) is not True

    @staticmethod
    @pytest.mark.c_ares
    def test_0008_set_dyndns_update_ptr_to_false_ptr_records_are_present(
            multihost, adjoin, extra_interface, extra_network):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: set dyndns update ptr to false ptr records are present
        Will test parameter dyndns_update_ptr = false when records exits
        NOTE: dyndns_refresh_interval lowest value is 60 seconds, with a 21 second timeout so the value is 81
        :id: 7c96e4d2-b1da-4391-9a7c-ff632667b1bc
        :steps:
        1. Join client to AD domain and delete all client DNS records
        2. Delete both A and PTR records
        3. Start SSSD
        4. Use DNS to check the host and IP
        5. Set dyndns_update_ptr = false and dyndns_refresh_interval = 81 in sssd.conf
        6. Update IP on extra interface and start SSSD
        7. Use DNS to check the host and IP
        :expectedresults:
        1. Client joins domain
        2. Host records are removed from the DNS server
        3. SSSD is started
        4. Host and IP is resolvable
        5. SSSD is stopped and sssd.conf has been updated
        6. IP on dummy has been changed
        7. Host and IP is resolvable but the PTR record is the old IP
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        domain = multihost.ad[0].domainname
        (extra_int, ip) = extra_interface
        ptr = str(random.randrange(2, 255))
        while ptr == str(ip.split(".")[3]):
            ptr = str(random.randrange(2, 255))
        new_ip = str(ip.split(".")[0]) + "." + str(ip.split(".")[1]) + "." + str(ip.split(".")[2]) + "." + ptr

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_refresh_interval': '81', 'dyndns_update': 'true', 'dyndns_iface': extra_int}
        client.sssd_conf(domain_section, sssd_params)
        dns.del_record(hostname)
        dns.del_record(ip)
        client.clear_sssd_cache()
        assert dns.find_a(hostname, ip)
        assert dns.find_ptr(hostname, ip)

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_update_ptr': 'false'}
        client.sssd_conf(domain_section, sssd_params)
        multihost.client[0].run_command('ip addr flush dev ' + extra_int)
        multihost.client[0].run_command('ip link del ' + extra_int)
        multihost.client[0].run_command('ip link add ' + extra_int + ' type dummy')
        multihost.client[0].run_command('ip addr add ' + new_ip + ' dev ' + extra_int)
        client.clear_sssd_cache()

        assert dns.find_a(hostname, new_ip)
        assert dns.find_ptr(hostname, ip)
        assert dns.find_ptr(hostname, new_ip) is not True

    @staticmethod
    @pytest.mark.c_ares
    def test_0009_check_with_dyndns_force_tcp(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: check with dyndns force tcp
        Test SSSD to use TCP for dynamic DNS. Blocking all TCP traffic will test that no other protocols will
        be used to update DNS
        :id: 0de1b45c-cecb-451f-b625-823761f255b2
        :steps:
        1. Join client to AD domain and delete all client DNS records
        2. Set ldap_purge_cache_timeout = 0, krb5_auth_timeout = 12, dyndns_force_tcp = true
        3. Update firewall to block TCP traffic on port 53
        4. Start SSSD
        5. Check DNS records
        6. Unblock traffic
        7. Check DNS records again
        :expectedresults:
        1. Client joins domain
        2. Configure SSSD
        3. Traffic is being blocked and dyndns is only using tcp
        4. SSSD starts
        5.  No A or PTR records exists
        6. Traffic is flowing
        7. DNS records are now updating properly
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname_cmd = multihost.client[0].run_command('hostname')
        hostname = hostname_cmd.stdout_text.rstrip()
        domain = multihost.ad[0].domainname
        ip = multihost.client[0].ip

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'ldap_purge_cache_timeout': '0', 'krb5_auth_timeout': '12', 'dyndns_force_tcp': 'true'}
        client.sssd_conf(domain_section, sssd_params)
        dns.del_record(hostname)
        dns.del_record(ip)
        multihost.client[0].run_command('which iptables || yum install -y iptables', raiseonerr=False)
        multihost.client[0].run_command(f'iptables -A INPUT -p tcp --dport 53 -s {multihost.ad[0].ip} -j DROP; '
                                        f'iptables -A OUTPUT -p tcp --dport 53 -d {multihost.ad[0].ip} -j DROP')
        client.clear_sssd_cache()
        assert dns.find_a(hostname, ip) is not True
        assert dns.find_ptr(hostname, ip) is not True

        multihost.client[0].run_command('iptables -F', raiseonerr=False)
        client.clear_sssd_cache()
        assert dns.find_a(hostname, ip)
        assert dns.find_ptr(hostname, ip)

    @staticmethod
    @pytest.mark.c_ares
    def test_0010_check_with_combination_of_addresses(
            multihost, adjoin, extra_interface, extra_network):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: check with combination of addresses
        Check DynDNS when the client has several interfaces, testing that the right interface updates DNS.
        NOTE: The original test includes IPV6, however CI does not support IPV6, so this is done with just IPV4
        :id: fc7e3b4d-5cd3-4206-b16d-d90c96acca14
        :steps:
        1. Join client to AD domain and delete all client DNS records
        2. Add extra interface and network
        3. Configure SSSD with dyndns_iface = extra_interface
        4. Start SSSD
        5. Check DNS records
        :expectedresults:
        1. Client joins domain
        2. Extra interface exists on client machine
        3. SSSD is configured
        4. SSSD starts
        5. DNS server will update with only the A and PTR records from the extra interface
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        domain = multihost.ad[0].domainname
        (extra_int, extra_ip) = extra_interface
        ip = multihost.client[0].ip

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_iface': f'{extra_int}'}
        client.sssd_conf(domain_section, sssd_params)
        dns.del_record(hostname)
        dns.del_record(ip)
        client.clear_sssd_cache()

        assert dns.find_a(hostname, extra_ip)
        assert dns.find_ptr(hostname, extra_ip)
        assert dns.find_ptr(hostname, ip) is not True

    @staticmethod
    def test_0011_verify_use_after_free_in_dyndns_code_bz1132361(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: verify use after free in dyndns code bz1132361
        bz1132361 is an OOM error that is recreated by
        setting TALLOC_FREE_FILL
        :id: 72d5e4ad-fdb9-4278-860f-57ff563a20d7
        :steps:
        1. Join client to AD domain and delete all client DNS records
        2. Add extra interface
        3. Set TALLOC_FREE_FILL=253 in /etc/sysconfig/sssd
        4. Start SSSD
        :expectedresults:
        1. Client joins domain
        2. Extra interface is created
        3. Configuration files updated
        4. SSSD starts and does not segfault
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        multihost.client[0].run_command('echo "TALLOC_FREE_FILL=253" >> /etc/sysconfig/sssd')
        client.clear_sssd_cache()

        ps_cmd = multihost.client[0].run_command('ps aux | grep sssd')
        assert 'sssd_be' in ps_cmd.stdout_text
        assert 'sssd_nss' in ps_cmd.stdout_text
        assert 'sssd_pam' in ps_cmd.stdout_text
        assert 'sssd_pac' in ps_cmd.stdout_text
        multihost.client[0].run_command('sed -i "/TALLOC_FREE_FILL/d" /etc/sysconfig/sssd')

    @staticmethod
    def test_0012_set_dyndns_update_ptr_when_dyndns_server_equals_ad_server(multihost, adjoin):
        """
        :title: IDM-SSSD-TC: ad_provider: dyndns: set dyndns update ptr when dyndns server equals ad server
        Testing parameter ad_server = $AD_SERVER
        :id: afa91b9b-4943-423a-a2e2-54f6c6204107
        :steps:
        1. Join client to AD domain and delete all client DNS records
        2. Configure SSSD to contain dyndns_server = AD_SERVER
        3. Start SSSD
        4. Check DNS records
        :expectedresults:
        1. Client joins domain
        2. SSSD is configured
        3. SSSD is started
        4. Both A and PTR records are found
        """
        adjoin(membersw='adcli')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        dns = ADDNS(multihost.ad[0])
        hostname = multihost.client[0].run_command('hostname').stdout_text.rstrip()
        domain = multihost.ad[0].domainname
        adserver = multihost.ad[0].hostname
        ip = multihost.client[0].ip

        domain_section = 'domain/{}'.format(domain)
        sssd_params = {'dyndns_server': f'{adserver}'}
        client.sssd_conf(domain_section, sssd_params)
        dns.del_record(hostname)
        dns.del_record(ip)
        client.clear_sssd_cache()
        assert dns.find_a(hostname, ip)
        assert dns.find_ptr(hostname, ip)



"""Automation for Krb access provider tests ported from bash

:requirement: krb_access_provider
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
import pytest
import time
from constants import ds_instance_name
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.ssh2_python import check_login_client, \
    check_login_client_bool, run_command_client, SSHClient


@pytest.fixture(scope='class')
def custom_setup(multihost, setup_sssd_krb, create_posix_usersgroups, krb_connection_timeout):
    """ Added neccessary sssd domain parameters """
    tools = sssdTools(multihost.client[0])
    sssd_params = {'sbus_timeout': 30,
                   'services': "nss, pam"}
    tools.sssd_conf('sssd', sssd_params)
    domain_section = f'domain/{ds_instance_name}'
    domain_params = {'access_provider': 'krb5',
                     'use_fully_qualified_names': 'False',
                     'override_homedir': '/home/%u'}
    tools.sssd_conf(domain_section, domain_params)
    tools.clear_sssd_cache()

    cmds = ["mkdir -p /home/foo3",
            "chown foo3: /home/foo3",
            "chmod 700 /home/foo3",
            "truncate -s 0 /home/foo3/.k5login",
            "chown foo3: /home/foo3/.k5login",
            "restorecon -v /home/foo3/.k5login"]
    for command in cmds:
        cmd = multihost.client[0].run_command(command, raiseonerr=False)
        assert cmd.returncode == 0, f'{command} did not execute successfully'


@pytest.mark.tier2
@pytest.mark.krbaccessprovider
@pytest.mark.usefixtures('custom_setup')
class TestKrbAccessProvider():
    """
    This is test case class for krb_access_provider suite

    Test access provision by kerberos on the basis of .k5login file.
    Test for empty k5login file and test for presence of single or
    multiple kerberos principal in the file, test for deleted user with
    principals present in the file.
    """
    @staticmethod
    def test_0001_k5login_empty(multihost):
        """
        :title: IDM-SSSD-TC: krb_provider: krb_access_provider: k5login is an empty file
        :id: e84bd024-669f-40d0-8833-edbc1be48e8e
        :setup:
          1. Clear the secure log, clear cache and restart sssd.
        :steps:
          1. Authenticate the user foo3 and foo4 from the client
          2. Check the secure log for expected messages.
          3. Check the sssd domain log for expected messages.
        :expectedresults:
          1. User foo3 should be able to successfully login
             User foo4 should not able to login
          2. Secure Log contains the expected lines:
             pam_sss(sshd:auth): authentication success for foo3
             pam_sss(sshd:account): Access denied for user foo3
             pam_sss(sshd:auth): authentication success for foo4
             Accepted password for foo4
          3. SSSD Domain Log contains the expected lines:
             Access denied for user [foo3
             Access allowed for user [foo4
        :teardown:
          1. Clear the .k5login file.
        """
        multihost.client[0].run_command("truncate -s 0 /var/log/secure", raiseonerr=False)
        client = sssdTools(multihost.client[0])
        client.clear_sssd_cache()
        with pytest.raises(Exception):
            check_login_client(multihost, "foo3", "Secret123")
        ssh = check_login_client_bool(multihost, "foo4", "Secret123")
        time.sleep(3)
        secure_log_str = multihost.client[0].get_file_contents("/var/log/secure").decode('utf-8')
        file = f'/var/log/sssd/sssd_{ds_instance_name}.log'
        sssd_log_str = multihost.client[0].get_file_contents(file).decode('utf-8')

        multihost.client[0].run_command("truncate -s 0 /home/foo3/.k5login", raiseonerr=False)
        assert ssh, 'foo4 is not able to login.'
        assert "pam_sss(sshd:auth): authentication success" in secure_log_str, \
               "authentication success not found in /var/log/secure"
        assert "pam_sss(sshd:account): Access denied for user foo3" in secure_log_str, \
               "Access denied not found in /var/log/secure"

        assert "pam_sss(sshd:auth): authentication success" in secure_log_str, \
               "authentication success not found in /var/log/secure"
        assert "Accepted password for foo4" in secure_log_str, "Accepted password not found in /var/log/secure"

        assert f"Access denied for user [foo3@{ds_instance_name}]" in sssd_log_str, \
               "Access denied not found in /var/log/sssd/sssd_example1.log"
        assert f"Access allowed for user [foo4@{ds_instance_name}]" in sssd_log_str, \
               "Access allowed not found in /var/log/sssd/sssd_example1.log"

    @staticmethod
    def test_0002_k5login_user3(multihost):
        """
        :title: IDM-SSSD-TC: krb_provider: krb_access_provider: k5login has testuser3
        :id: decede10-c500-4b56-8037-7f2e325009b6
        :setup:
          1. Clear the secure log, clear cache and restart sssd.
          2. Add the foo3 user principal in the k5login file
        :steps:
          1. Authenticate the user foo3 from the client
          2. Check the secure log for expected messages.
          3. Check the sssd domain log for expected messages.
        :expectedresults:
          1. User foo3 should be able to successfully login
          2. Secure Log contains the expected lines:
             pam_sss(sshd:auth): authentication success for foo3
             Accepted password for foo3
          3. SSSD Domain Log contains the expected lines:
             Access allowed for user [foo3
        :teardown:
          1. Clear the .k5login file.
        """
        multihost.client[0].run_command("truncate -s 0 /var/log/secure", raiseonerr=False)
        client = sssdTools(multihost.client[0])
        client.clear_sssd_cache()

        cmds = ['echo "foo3@EXAMPLE.TEST" > /home/foo3/.k5login',
                "restorecon -v /home/foo3/.k5login"]
        for command in cmds:
            cmd = multihost.client[0].run_command(command, raiseonerr=False)

        ssh = check_login_client_bool(multihost, "foo3", "Secret123")
        time.sleep(3)
        secure_log_str = multihost.client[0].get_file_contents("/var/log/secure").decode('utf-8')

        file = f'/var/log/sssd/sssd_{ds_instance_name}.log'
        sssd_log_str = multihost.client[0].get_file_contents(file).decode('utf-8')

        multihost.client[0].run_command("truncate -s 0 /home/foo3/.k5login", raiseonerr=False)
        assert ssh, 'foo3 is not able to login.'
        assert cmd.returncode == 0, f"{command} did not execute successfully"
        assert "pam_sss(sshd:auth): authentication success" in secure_log_str, \
               "authentication success not found in /var/log/secure"
        assert "Accepted password for foo3" in secure_log_str, \
               "Accepted password not found in /var/log/secure"
        assert f"Access allowed for user [foo3@{ds_instance_name}]" in sssd_log_str, \
               "Access allowed not found in /var/log/sssd/sssd_example1.log"

    @staticmethod
    def test_0003_k5login_user3_with_user4(multihost):
        """
        :title: IDM-SSSD-TC: krb_provider: krb_access_provider: k5login has testuser3 and testuser4
        :id: 3ddb4948-6b5a-4e82-85f7-c896bce14dda
        :setup:
          1. Clear the secure log, clear cache and restart sssd.
          2. Add both the user principal in the k5login file
        :steps:
          1. Authenticate the user foo3 from the client
          2. Check the /tmp/accessprovider.out file for expected messages.
        :expectedresults:
          1. User foo3 should be able to successfully login
          2. /tmp/accessprovider.out file contains the expected output:
             'foo3' username should be present in the file.
        :teardown:
          1. Clear the .k5login file.
          2. Remove the /tmp/accessProvider.out file
        """
        multihost.client[0].run_command("truncate -s 0 /var/log/secure", raiseonerr=False)
        client = sssdTools(multihost.client[0])
        client.clear_sssd_cache()

        echo_cmds = ['echo "foo3@EXAMPLE.TEST" > /home/foo3/.k5login',
                     'echo "foo4@EXAMPLE.TEST" >> /home/foo3/.k5login',
                     'restorecon -v /home/foo3/.k5login']

        for command in echo_cmds:
            cmd = multihost.client[0].run_command(command, raiseonerr=False)

        ssh = check_login_client_bool(multihost, "foo3", "Secret123")
        client_hostname = multihost.client[0].sys_hostname
        try:
            run_command_client(multihost, "foo4", "Secret123",
                               f"ssh -o StrictHostKeyChecking=no -o "
                               f"PasswordAuthentication=no foo3@{client_hostname} "
                               f"id > /tmp/accessProvider_id_krb5_003.out 2>&1")
        except Exception:
            pytest.fail("Error in connection via ssh as foo4")
        finally:
            multihost.client[0].run_command("truncate -s 0 /home/foo3/.k5login", raiseonerr=False)
        logfile = multihost.client[0].get_file_contents("/tmp/accessProvider_id_krb5_003.out").decode('utf-8')
        multihost.client[0].run_command("rm -rf /tmp/accessProvider_id_krb5_003.out", raiseonerr=False)
        assert ssh, 'foo3 is not able to login.'
        assert cmd.returncode == 0, f"{command} did not execute successfully"
        assert "foo3" in logfile, "foo3 not found in /tmp/accessProvider_id_krb5_003.out"

    @staticmethod
    def test_0004_k5login_user4_with_deleted_user3(multihost):
        """
        :title: IDM-SSSD-TC: krb_provider: krb_access_provider: k5login has testuser3 and testuser4\
           and testuser3 is deleted
        :id: ff46741d-cab8-4217-840a-fcd72857b0a6
        :setup:
          1. Clear the secure log, clear cache and restart sssd.
          2. Add both the user principal in the k5login file
        :steps:
          1. Authenticate the user foo3 from the client
          2. Delete the ldapuser foo3
          3. Check the /tmp/accessprovider.out file for expected messages.
        :expectedresults:
          1. User foo3 should be able to successfully login
          2. ldapuser foo3 should be successfully deleted
          3. /tmp/accessprovider.out file contains the expected output:
             'foo4' username should be present in the file.
             'foo3' username should not be present in the file.
        :teardown:
          1. Clear the .k5login file.
          2. Remove the /tmp/accessProvider.out file
        """
        multihost.client[0].run_command("truncate -s 0 /var/log/secure", raiseonerr=False)
        client = sssdTools(multihost.client[0])
        client.clear_sssd_cache()

        echo_cmds = ['echo "foo3@EXAMPLE.TEST" > /home/foo3/.k5login',
                     'echo "foo4@EXAMPLE.TEST" >> /home/foo3/.k5login',
                     'restorecon -v /home/foo3/.k5login']
        for command in echo_cmds:
            cmd1 = multihost.client[0].run_command(command, raiseonerr=False)

        ssh1 = check_login_client_bool(multihost, "foo3", "Secret123")

        ldap_cmd = f'ldapdelete -x -H ldap://{multihost.master[0].sys_hostname} \
                   -D "cn=Directory Manager" -w "Secret123" uid=foo3,ou=People,dc=example,dc=test'
        cmd2 = multihost.client[0].run_command(ldap_cmd, raiseonerr=False)
        time.sleep(10)
        client_hostname = multihost.client[0].sys_hostname
        try:
            ssh = SSHClient(client_hostname, "foo4", "Secret123")
            ssh.connect()
            ssh.execute_command("id > /tmp/accessProvider_id_krb5_004.out 2>&1")
            ssh.execute_command(f"ssh -o StrictHostKeyChecking=no -o "
                                f"PasswordAuthentication=no foo3@{client_hostname} id >> "
                                f"/tmp/accessProvider_id_krb5_004.out 2>&1")
            ssh.close()
        except Exception:
            pytest.fail("Error in connection via ssh as foo4")
        finally:
            multihost.client[0].run_command("truncate -s 0 /home/foo3/.k5login", raiseonerr=False)
        logfile = multihost.client[0].get_file_contents("/tmp/accessProvider_id_krb5_004.out").decode('utf-8')
        multihost.client[0].run_command("rm -rf /tmp/accessProvider_id_krb5_004.out", raiseonerr=False)
        assert ssh1, 'foo1 is not able to login.'
        assert cmd1.returncode == 0, f'{command} did not execute successfully'
        assert cmd2.returncode == 0, f'{ldap_cmd} did not execute successfully'
        assert "foo3" not in logfile, "foo3 found in /tmp/accessProvider_id_krb5_004.out"
        assert "foo4" in logfile, "foo4 not found in /tmp/accessProvider_id_krb5_004.out"
